@@ -123,7 +123,7 @@ void FsmAp::allTransPrior( int ordering, PriorDesc *prior )
  * transitions that may go into those states in the future. The graph does not
  * support pending in transitions in the same way pending out transitions are
  * supported. */
-void FsmAp::finFsmPrior( int ordering, PriorDesc *prior )
+void FsmAp::finishFsmPrior( int ordering, PriorDesc *prior )
 {
 	/* Walk all final states. */
 	for ( StateSet::Iter state = finStateSet; state.lte(); state++ ) {
@@ -189,7 +189,7 @@ void FsmAp::allTransAction( int ordering, int actionId )
  * is final we can't really specify a function to execute upon entering that
  * final state the first time. So function really means whenever entering a
  * final state from within the same fsm. */
-void FsmAp::finFsmAction( int ordering, int actionId )
+void FsmAp::finishFsmAction( int ordering, int actionId )
 {
 	/* Walk all final states. */
 	for ( StateSet::Iter state = finStateSet; state.lte(); state++ ) {
@@ -277,7 +277,7 @@ void FsmAp::allErrorAction( int ordering, int actionId, int transferPoint )
 }
 
 /* Set error actions in the states that have transitions into a final state. */
-void FsmAp::finErrorAction( int ordering, int actionId, int transferPoint )
+void FsmAp::finishErrorAction( int ordering, int actionId, int transferPoint )
 {
 	/* Isolate the start state in case it is reachable from in inside the
 	 * machine, in which case we don't want it set. */
@@ -295,6 +295,39 @@ void FsmAp::leaveErrorAction( int ordering, int actionId, int transferPoint )
 		(*state)->errActionTable.setAction( ordering, actionId, transferPoint );
 }
 
+void FsmAp::startContext( int contextId )
+{
+	/* Make sure the start state has no other entry points. */
+	isolateStartState();
+
+	/* Add the actions. */
+	startState->contextSet.insert( contextId );
+}
+
+void FsmAp::allContext( int contextId )
+{
+	/* Insert actions in the error action table of all states. */
+	for ( StateList::Iter state = stateList; state.lte(); state++ )
+		state->contextSet.insert( contextId );
+}
+
+void FsmAp::finishContext( int contextId )
+{
+	/* Isolate the start state in case it is reachable from in inside the
+	 * machine, in which case we don't want it set. */
+	for ( StateList::Iter state = stateList; state.lte(); state++ ) {
+		if ( state != startState && ! state->isFinState() )
+			state->contextSet.insert( contextId );
+	}
+}
+
+void FsmAp::leaveContext( int contextId )
+{
+	/* Add the action to the error table of final states. */
+	for ( StateSet::Iter state = finStateSet; state.lte(); state++ )
+		(*state)->contextSet.insert( contextId );
+}
+
 /* Shift the function ordering of the start transitions to start
  * at fromOrder and increase in units of 1. Useful before staring.
  * Returns the maximum number of order numbers used. */
@@ -502,33 +535,6 @@ void FsmAp::leavingFromState( TransAp *destTrans, StateAp *srcState )
 }
 
 
-/* Callback invoked when otherState is added into this state during the
- * merging process.  Draw in any properties of srcState into this state.
- * AddInState is called when a new state is made that will be a duplicate of
- * another state or a combination of several other states. AddInState will be
- * called for each state that the new state is to represent. */
-void FsmAp::addInState( StateAp *destState, StateAp *srcState )
-{
-	/* Merge the outTransFuncTable. */
-	if ( srcState == destState ) {
-		/* Duplicate the list to protect against write to source. The
-		 * priorities are not copied in as that would have no effect. */
-		destState->outActionTable.setActions( ActionTable( 
-				srcState->outActionTable ) );
-		destState->errActionTable.setActions( ErrActionTable( 
-				srcState->errActionTable ) );
-		destState->eofActionTable.setActions( ActionTable( 
-				srcState->eofActionTable ) );
-	}
-	else {
-		/* Get the out actions, out priorities and error actions. */
-		destState->outActionTable.setActions( srcState->outActionTable );
-		destState->errActionTable.setActions( srcState->errActionTable );
-		destState->eofActionTable.setActions( srcState->eofActionTable );
-		destState->outPriorTable.setPriors( srcState->outPriorTable );
-	}
-}
-
 /* Callback invoked when a state looses its final state status. This is where
  * properties of final states get unset. At the point of invocation, it is
  * unspecified whether or not isFinState is set.  Callback invoked when a
