@@ -233,9 +233,11 @@ void FsmAp::detachState( StateAp *state )
 		unsetEntry( state->entryIds[0], state );
 
 	/* Detach out range transitions. */
-	for ( TransList::Iter trans = state->outList; trans.lte(); trans++ ) {
+	for ( TransList::Iter trans = state->outList; trans.lte(); ) {
+		TransList::Iter next = trans.next();
 		detachTrans( state, trans->toState, trans, KeyTypeRange );
 		delete trans;
+		trans = next;
 	}
 
 	/* Delete all of the out range pointers. */
