@@ -589,14 +589,15 @@ void FsmAp::epsilonOp()
 	for ( StateList::Iter st = stateList; st.lte(); st++ )
 		st->owningGraph = 0;
 
-	/* Perform merges. Epsilon transitions without a join will only every
+	/* Perform merges. Epsilon transitions without a join will only ever
 	 * increase connectivity. No unreachable or dead end states should be
 	 * created. */
 	resolveEpsilonTrans( md );
 }
 
 /* Make a new maching by joining together a bunch of machines without making
- * any transitions between them. */
+ * any transitions between them. A negative finalId results in there being no
+ * final id. */
 void FsmAp::joinOp( int startId, int finalId, FsmAp **others, int numOthers )
 {
 	/* For the merging process. */
@@ -670,15 +671,17 @@ void FsmAp::joinOp( int startId, int finalId, FsmAp **others, int numOthers )
 	/* Now all final states are unset. */
 	unsetAllFinStates();
 
-	/* Create the implicit final state. */
-	StateAp *finState = addState();
-	setFinState( finState );
+	if ( finalId >= 0 ) {
+		/* Create the implicit final state. */
+		StateAp *finState = addState();
+		setFinState( finState );
 
-	/* Assign an entry into the final state on the final state entry id. Note
-	 * that there may already be an entry on this id. That's ok. Also set the
-	 * final state owning machine id. It's in a class all it's own. */
-	setEntry( finalId, finState );
-	finState->owningGraph = 0;
+		/* Assign an entry into the final state on the final state entry id. Note
+		 * that there may already be an entry on this id. That's ok. Also set the
+		 * final state owning machine id. It's in a class all it's own. */
+		setEntry( finalId, finState );
+		finState->owningGraph = 0;
+	}
 
 	/* Hand over to workers for resolving epsilon trans. This will merge states
 	 * with the targets of their epsilon transitions. */
@@ -863,14 +866,25 @@ void FsmAp::mergeStates( MergeData &md, StateAp *destState,
 	if ( srcState->stateBits & SB_ISFINAL )
 		setFinState( destState );
 
-	/* Add in epsilon transitions. */
-	if ( destState == srcState )
+	/* Draw in any properties of srcState into destState. */
+	if ( srcState == destState ) {
+		/* Duplicate the list to protect against write to source. The
+		 * priorities and context sets are not copied in because that would
+		 * have no effect. */
 		destState->epsilonTrans.append( EpsilonTrans( srcState->epsilonTrans ) );
-	else 
+		destState->outActionTable.setActions( ActionTable( srcState->outActionTable ) );
+		destState->errActionTable.setActions( ErrActionTable( srcState->errActionTable ) );
+		destState->eofActionTable.setActions( ActionTable( srcState->eofActionTable ) );
+	}
+	else {
+		/* Get the epsilons, out actions, out priorities and error actions. */
 		destState->epsilonTrans.append( srcState->epsilonTrans );
-
-	/* Call user routine. */
-	addInState( destState, srcState );
+		destState->contextSet.insert( srcState->contextSet );
+		destState->outActionTable.setActions( srcState->outActionTable );
+		destState->errActionTable.setActions( srcState->errActionTable );
+		destState->eofActionTable.setActions( srcState->eofActionTable );
+		destState->outPriorTable.setPriors( srcState->outPriorTable );
+	}
 }
 
 void FsmAp::mergeStates( MergeData &md, StateAp *destState, 
