@@ -440,7 +440,10 @@ struct EptVectEl
 typedef Vector<EptVectEl> EptVect;
 
 /* Set of entry ids that go into this state. */
-typedef BstSet< int > EntryIdSet;
+typedef BstSet<int> EntryIdSet;
+
+/* Set of context ids. */
+typedef BstSet<int> ContextSet;
 
 /* State class that implements actions and priorities. */
 struct StateAp 
@@ -465,6 +468,9 @@ struct StateAp
 	/* Epsilon transitions. */
 	EpsilonTrans epsilonTrans;
 
+	/* Context set in the state. */
+	ContextSet contextSet;
+
 	/* Number of in transitions from states other than ourselves. */
 	int foreignInTrans;
 
@@ -662,6 +668,10 @@ private:
 /* Compare lists of epsilon transitions. Entries are name ids of targets. */
 typedef CmpTable< int, CmpOrd<int> > CmpEpsilonTrans;
 
+/* Compare sets of context values. */
+typedef CmpTable< int, CmpOrd<int> > CmpContextSets;
+
+
 /* Compare class for the Approximate minimization. */
 class ApproxCompare
 {
@@ -754,7 +764,7 @@ struct FsmAp
 	/* Set priorities on transtions. */
 	void startFsmPrior( int ordering, PriorDesc *prior );
 	void allTransPrior( int ordering, PriorDesc *prior );
-	void finFsmPrior( int ordering, PriorDesc *prior );
+	void finishFsmPrior( int ordering, PriorDesc *prior );
 	void leaveFsmPrior( int ordering, PriorDesc *prior );
 
 	/* Action setting support. */
@@ -765,15 +775,21 @@ struct FsmAp
 	/* Set actions to execute. */
 	void startFsmAction( int ordering, int actionId );
 	void allTransAction( int ordering, int actionId );
-	void finFsmAction( int ordering, int actionId );
+	void finishFsmAction( int ordering, int actionId );
 	void leaveFsmAction( int ordering, int actionId );
 
 	/* Set error actions to execute. */
 	void startErrorAction( int ordering, int actionId, int transferPoint );
 	void allErrorAction( int ordering, int actionId, int transferPoint );
-	void finErrorAction( int ordering, int actionId, int transferPoint );
+	void finishErrorAction( int ordering, int actionId, int transferPoint );
 	void leaveErrorAction( int ordering, int actionId, int transferPoint );
 
+	/* Set Context. */
+	void startContext( int contextId );
+	void allContext( int contextId );
+	void finishContext( int contextId );
+	void leaveContext( int contextId );
+
 	/* Shift the action ordering of the start transitions to start at
 	 * fromOrder and increase in units of 1. Useful before kleene star
 	 * operation.  */
@@ -806,6 +822,8 @@ struct FsmAp
 	void unsetEntry( int id, StateAp *state );
 	void unsetEntry( int id );
 	void unsetAllEntryPoints();
+
+	/* Epsilon transitions. */
 	void epsilonTrans( int id );
 	void shadowReadWriteStates( MergeData &md );
 
@@ -922,9 +940,6 @@ struct FsmAp
 	/* This trans is going to be made a leaving-graph-trans from srcState. */
 	void leavingFromState( TransAp *destTrans, StateAp *srcState );
 	
-	/* Invoked when otherState is added into this state during the merging process. */
-	void addInState( StateAp *destState, StateAp *srcState );
-
 	/* Invoked when a state looses its final state status. */
 	void relinquishFinal( StateAp *state );
 
