@@ -141,6 +141,7 @@ StateAp::StateAp()
 	/* No entry points, or epsilon trans. */
 	entryIds(),
 	epsilonTrans(),
+	contextSet(),
 
 	/* No transitions in from other states. */
 	foreignInTrans(0),
@@ -164,20 +165,18 @@ StateAp::StateAp()
  * FsmAp copy constructor. */
 StateAp::StateAp(const StateAp &other)
 :
-	/* All in lists are cleared. They will be filled in when the
+	/* All lists are cleared. They will be filled in when the
 	 * individual transitions are duplicated and attached. */
 	outList(),
 	outDefault(0),
 	inRange(),
 	inDefault(),
 
-	/* Duplicate the entry id set. This is a set of integers 
-	 * and as such needs no fixing. */
+	/* Duplicate the entry id set, epsilon transitions and context sets. These
+	 * are sets of integers and as such need no fixing. */
 	entryIds(other.entryIds),
-
-	/* Copy the out pointers directly. 
-	 * The transitions will be duplicated later. */
 	epsilonTrans(other.epsilonTrans),
+	contextSet(other.contextSet),
 
 	/* No transitions in from other states. */
 	foreignInTrans(0),
@@ -233,6 +232,12 @@ int ApproxCompare::compare( const StateAp *state1 , const StateAp *state2 )
 	if ( compareRes != 0 )
 		return compareRes;
 	
+	/* Test context sets. */
+	compareRes = CmpContextSets::compare( state1->contextSet, 
+			state2->contextSet );
+	if ( compareRes != 0 )
+		return compareRes;
+	
 	/* Compare the out transitions. */
 	compareRes = FsmAp::compareStateData( state1, state2 );
 	if ( compareRes != 0 )
@@ -298,6 +303,12 @@ int InitPartitionCompare::compare( const StateAp *state1 , const StateAp *state2
 	if ( compareRes != 0 )
 		return compareRes;
 
+	/* Test context sets. */
+	compareRes = CmpContextSets::compare( state1->contextSet, 
+			state2->contextSet );
+	if ( compareRes != 0 )
+		return compareRes;
+
 	/* Compare the out transitions. */
 	compareRes = FsmAp::compareStateData( state1, state2 );
 	if ( compareRes != 0 )
