@@ -41,7 +41,10 @@ std::ostream &GotoCodeGen::TRANS_GOTO( RedTransAp *trans, int level )
 
 std::ostream &GotoCodeGen::FIRST_FINAL_STATE()
 {
-	out << redFsm->firstFinState->id;
+	if ( redFsm->firstFinState != 0 )
+		out << redFsm->firstFinState->id;
+	else
+		out << redFsm->nextStateId;
 	return out;
 }
 
@@ -58,13 +61,13 @@ std::ostream &GotoCodeGen::ACTION_SWITCH()
 		/* Write out referenced actions. */
 		if ( act->numTransRefs > 0 || act->numEofRefs > 0 ) {
 			/* Get the function data. Print the case label.  */
-			out << "\tcase " << act->id << ":\n";
+			out << "\tcase " << act->actionId << ":\n";
 
 			/* Write the preprocessor line info for going into the source file. */
 			out << "#line " << act->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
 
 			/* Write the block and close it off. */
-			out << "\t{"; ACTION(act, 0, false) << "} break;\n";
+			out << "\t{"; INLINE_LIST(act->inlineList, 0, false) << "} break;\n";
 		}
 	}
 
@@ -222,7 +225,7 @@ std::ostream &GotoCodeGen::TRANSITIONS()
 
 		if ( trans->action != 0 ) {
 			/* Write out the transition func. */
-			out << "goto f" << trans->action->id << ";\n";
+			out << "goto f" << trans->action->actListId << ";\n";
 		}
 		else {
 			/* No code to execute, just loop around. */
@@ -243,7 +246,7 @@ std::ostream &GotoCodeGen::EXEC_FUNCS()
 {
 	/* Make labels that set acts and jump to execFuncs. Loop func indicies. */
 	for ( ActionTableMap::Iter act = redFsm->actionMap; act.lte(); act++ ) {
-		out << "\tf" << act->id << ": _acts = "; FSM_NAME() << "_f+" << 
+		out << "\tf" << act->actListId << ": _acts = "; FSM_NAME() << "_a+" << 
 				act->location+1 << "; goto execFuncs;\n";
 	}
 
@@ -269,7 +272,7 @@ std::ostream &GotoCodeGen::FINISH_CASES()
 			out << "\t\tcase " << st->id << ": ";
 
 			/* Write the goto func. */
-			out << "goto f" << st->eofAction->id << ";\n";
+			out << "goto f" << st->eofAction->actListId << ";\n";
 		}
 	}
 	
@@ -284,9 +287,11 @@ std::ostream &GotoCodeGen::GOTO( NameInst *name, bool inFinish )
 	return out;
 }
 
-std::ostream &GotoCodeGen::GOTOE( char *stateExpr, bool inFinish )
+std::ostream &GotoCodeGen::GOTOE( InlineItem *ilItem, bool inFinish )
 {
-	out << "{_cs = (" << stateExpr << "); goto again;}";
+	out << "{_cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << "); goto again;}";
 	return out;
 }
 
@@ -309,7 +314,7 @@ std::ostream &GotoCodeGen::TARGS( bool inFinish, int targState )
 	return out;
 }
 
-std::ostream &GotoCodeGen::GET_STATE( NameInst *name, bool inFinish )
+std::ostream &GotoCodeGen::GET_ENTRY( NameInst *name, bool inFinish )
 {
 	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
 	out << targ->value->id;
@@ -323,12 +328,43 @@ std::ostream &GotoCodeGen::NEXT( NameInst *name, bool inFinish )
 	return out;
 }
 
-std::ostream &GotoCodeGen::NEXTE( char *stateExpr, bool inFinish )
+std::ostream &GotoCodeGen::NEXTE( InlineItem *ilItem, bool inFinish )
 {
-	out << "_cs = (" << stateExpr << ");";
+	out << "_cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << ");";
 	return out;
 }
 
+std::ostream &GotoCodeGen::CONTEXT( int contextId )
+{
+	/* Make a vect to fill with context indexed by state. */
+	int numVals = (redFsm->nextStateId>>3)+1;
+	unsigned char *vals = new unsigned char[numVals];
+	memset( vals, 0, sizeof(unsigned char)*numVals );
+
+	/* Fill the context vect. */
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		if ( st->contextSet.find( contextId ) )
+			vals[st->id>>3] |= 1 << (st->id&0x7);
+	}
+
+	/* Write out the vect. */
+	for ( int i = 0; i < numVals; ) {
+		out << (unsigned int)vals[i];
+
+		i += 1;
+		if ( i < numVals ) {
+			out << ", ";
+			if ( i % IALL == 0 )
+				out << "\n\t";
+		}
+	}
+	out << "\n";
+
+	delete[] vals;
+	return out;
+}
 
 
 /* Init base data. */
@@ -353,34 +389,14 @@ std::ostream &CGotoCodeGen::RET( bool inFinish )
 	return out;
 }
 
-std::ostream &CGotoCodeGen::CALLE( char *stateExpr, int targState, bool inFinish )
+std::ostream &CGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
 {
-	out << "{fsm->_st[fsm->_top++] = _cs; _cs = (" << stateExpr << "); goto again;}";
+	out << "{fsm->_st[fsm->_top++] = _cs; _cs = (";
+	INLINE_LIST( ilItem->children, targState, inFinish );
+	out << "); goto again;}";
 	return out;
 }
 
-void CGotoCodeGen::writeOutHeader()
-{
-	out <<
-		"/* Only non-static data: current state. */\n"
-		"struct "; FSM_NAME() << "\n"
-		"{\n"
-		"	int _cs;\n";
-		STRUCT_DATA() <<
-		"};\n"
-		"\n"
-		"/* Init the fsm. */\n"
-		"int "; FSM_NAME() << "_init( struct "; FSM_NAME() << " *fsm );\n"
-		"\n"
-		"/* Execute some chunk of data. */\n"
-		"int "; FSM_NAME() << "_execute( struct "; FSM_NAME() << " *fsm, "; EL_TYPE() <<
-				" *data, int len );\n"
-		"\n"
-		"/* Indicate to the fsm tha there is no more data. */\n"
-		"int "; FSM_NAME() << "_finish( struct "; FSM_NAME() << " *fsm );\n"
-		"\n";
-}
-
 void CGotoCodeGen::writeOutCode()
 {
 	out <<
@@ -389,7 +405,7 @@ void CGotoCodeGen::writeOutCode()
 
 	if ( anyActions() ) {
 		out <<
-			"static int "; FSM_NAME() << "_f[] = {\n";
+			"static "; ARRAY_TYPE(maxActArrItem) << " "; FSM_NAME() << "_a[] = {\n";
 			ACTIONS_ARRAY() <<
 			"};\n"
 			"\n";
@@ -398,14 +414,14 @@ void CGotoCodeGen::writeOutCode()
 	out <<
 		"int "; FSM_NAME() << "_init( struct "; FSM_NAME() << " *fsm )\n"
 		"{\n"
-		"	fsm->_cs = "; FSM_NAME() << "_start;\n";
+		"	fsm->curs = "; FSM_NAME() << "_start;\n";
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
 		out << "	fsm->_top = 0;\n";
 
 	INIT_CODE() <<
-		"	if ( fsm->_cs >= "; FIRST_FINAL_STATE() << " )\n"
+		"	if ( fsm->curs >= "; FIRST_FINAL_STATE() << " )\n"
 		"		return 1;\n"
 		"	return 0;\n"
 		"}\n"
@@ -415,15 +431,20 @@ void CGotoCodeGen::writeOutCode()
 		"{\n"
 		"	"; EL_TYPE() << " *_p = _data-1;\n"
 		"	"; EL_TYPE() << " *_pe = _data+_len;\n"
-		"	int _cs = fsm->_cs";
+		"	int _cs = fsm->curs";
 
 	if ( anyRegCurStateRef() )
 		out << ", _ps = 0";
 	out << ";\n";
 
-	if ( anyActions() )
-		out << "\tint *_acts, _nacts;\n";
-	out << "\n";
+	if ( anyActions() ) {
+		out << "\t";
+		ARRAY_TYPE(maxActArrItem) << " *_acts, _nacts;\n";
+	}
+
+	out << 
+		"	fsm->curs = -1;\n"
+		"\n";
 
 	out << 
 		"again:\n"
@@ -460,7 +481,7 @@ void CGotoCodeGen::writeOutCode()
 	
 	out <<
 		"out:\n"
-		"	fsm->_cs = _cs;\n";
+		"	fsm->curs = _cs;\n";
 
 	if ( redFsm->errState != 0 ) {
 		out << 
@@ -487,12 +508,12 @@ void CGotoCodeGen::writeOutCode()
 	else {
 		if ( redFsm->errState != 0 ) {
 			out << 
-				"	if ( fsm->_cs == "; ERROR_STATE() << " )\n"
+				"	if ( fsm->curs == "; ERROR_STATE() << " )\n"
 				"		return -1;\n";
 		}
 
 		out <<
-			"	if ( fsm->_cs >= "; FIRST_FINAL_STATE() << " )\n"
+			"	if ( fsm->curs >= "; FIRST_FINAL_STATE() << " )\n"
 			"		return 1;\n"
 			"	return 0;\n";
 	}
@@ -500,16 +521,24 @@ void CGotoCodeGen::writeOutCode()
 	out <<
 		"}\n"
 		"\n";
+
+	for ( ContextMap::Iter ctx = parseData->contextMap; ctx.lte(); ctx++ ) {
+		/* Write out the vect. */
+		out << "unsigned char _"; FSM_NAME() << "_ctxdata_" << ctx->key << "[] = {\n\t";
+		CONTEXT( ctx->value->id );
+		out << "};\n\n";
+	}
 }
 
+
 /* Init base data. */
-CCGotoCodeGen::CCGotoCodeGen( char *fsmName, ParseData *parseData, 
+CppGotoCodeGen::CppGotoCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 : 
 	GotoCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &CCGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
+std::ostream &CppGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
 {
 	/* Lookup the target. Always guaranteed to return just one target. */
 	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
@@ -518,42 +547,21 @@ std::ostream &CCGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish
 	return out;
 }
 
-std::ostream &CCGotoCodeGen::RET( bool inFinish )
+std::ostream &CppGotoCodeGen::RET( bool inFinish )
 {
 	out << "{_cs = this->_st[--this->_top]; goto again;}";
 	return out;
 }
 
-std::ostream &CCGotoCodeGen::CALLE( char *stateExpr, int targState, bool inFinish )
+std::ostream &CppGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
 {
-	out << "{this->_st[this->_top++] = _cs; _cs = (" << 
-			stateExpr << "); goto again;}";
+	out << "{this->_st[this->_top++] = _cs; _cs = (";
+	INLINE_LIST( ilItem->children, targState, inFinish );
+	out << "); goto again;}";
 	return out;
 }
 
-void CCGotoCodeGen::writeOutHeader()
-{
-	out <<
-		"/* Only non-static data: current state. */\n"
-		"class "; FSM_NAME() << "\n"
-		"{\n"
-		"public:\n"
-		"	int _cs;\n"
-		"\n"
-		"	/* Init the fsm. */\n"
-		"	int init( );\n"
-		"\n"
-		"	/* Execute some chunk of data. */\n"
-		"	int execute( "; EL_TYPE() << " *data, int len );\n"
-		"\n"
-		"	/* Indicate to the fsm tha there is no more data. */\n"
-		"	int finish( );\n";
-		STRUCT_DATA() <<
-		"};\n"
-		"\n";
-}
-
-void CCGotoCodeGen::writeOutCode()
+void CppGotoCodeGen::writeOutCode()
 {
 	out <<
 		"static int "; FSM_NAME() << "_start = "; START_STATE_ID() << ";\n"
@@ -561,7 +569,7 @@ void CCGotoCodeGen::writeOutCode()
 
 	if ( anyActions() ) {
 		out <<
-			"static int "; FSM_NAME() << "_f[] = {\n";
+			"static "; ARRAY_TYPE(maxActArrItem) << " "; FSM_NAME() << "_a[] = {\n";
 			ACTIONS_ARRAY() <<
 			"};\n"
 			"\n";
@@ -570,14 +578,14 @@ void CCGotoCodeGen::writeOutCode()
 	out <<
 		"int "; FSM_NAME() << "::init( )\n"
 		"{\n"
-		"	this->_cs = "; FSM_NAME() << "_start;\n";
+		"	this->curs = "; FSM_NAME() << "_start;\n";
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
 		out << "	this->_top = 0;\n";
 
 	INIT_CODE() <<
-		"	if ( this->_cs >= "; FIRST_FINAL_STATE() << " )\n"
+		"	if ( this->curs >= "; FIRST_FINAL_STATE() << " )\n"
 		"		return 1;\n"
 		"	return 0;\n"
 		"}\n"
@@ -588,16 +596,20 @@ void CCGotoCodeGen::writeOutCode()
 		"{\n"
 		"	"; EL_TYPE() << " *_p = _data-1;\n"
 		"	"; EL_TYPE() << " *_pe = _data+_len;\n"
-		"	int _cs = this->_cs";
+		"	int _cs = this->curs";
 
 	if ( anyRegCurStateRef() )
 		out << ", _ps = 0";
 	out << ";\n";
 
-	if ( anyActions() )
-		out << "\tint *_acts, _nacts;\n";
+	if ( anyActions() ) {
+		out << "\t";
+		ARRAY_TYPE(maxActArrItem) << " *_acts, _nacts;\n";
+	}
 
-	out << "\n";
+	out << 
+		"	this->curs = -1;\n"
+		"\n";
 
 	out << 
 		"again:\n"
@@ -634,7 +646,7 @@ void CCGotoCodeGen::writeOutCode()
 
 	out <<
 		"out:\n"
-		"	this->_cs = _cs;\n";
+		"	this->curs = _cs;\n";
 
 	if ( redFsm->errState != 0 ) {
 		out <<
@@ -661,12 +673,12 @@ void CCGotoCodeGen::writeOutCode()
 	else {
 		if ( redFsm->errState != 0 ) {
 			out << 
-				"	if ( this->_cs == "; ERROR_STATE() << " )\n"
+				"	if ( this->curs == "; ERROR_STATE() << " )\n"
 				"		return -1;\n";
 		}
 
 		out <<
-			"	if ( this->_cs >= "; FIRST_FINAL_STATE() << " )\n"
+			"	if ( this->curs >= "; FIRST_FINAL_STATE() << " )\n"
 			"		return 1;\n"
 			"	return 0;\n";
 	}
@@ -674,4 +686,175 @@ void CCGotoCodeGen::writeOutCode()
 	out <<
 		"}\n"
 		"\n";
+
+	for ( ContextMap::Iter ctx = parseData->contextMap; ctx.lte(); ctx++ ) {
+		out << "unsigned char "; FSM_NAME() << "::_ctxdata_" << ctx->key << "[] = {\n\t";
+		CONTEXT( ctx->value->id );
+		out << "};\n\n";
+	}
 }
+
+
+/* Init base data. */
+ObjCGotoCodeGen::ObjCGotoCodeGen( char *fsmName, ParseData *parseData, 
+		RedFsmAp *redFsm, std::ostream &out )
+: 
+	GotoCodeGen(fsmName, parseData, redFsm, out)
+{ }
+
+std::ostream &ObjCGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
+{
+	/* Lookup the target. Always guaranteed to return just one target. */
+	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
+	out << "{self->_st[self->_top++] = _cs; _cs = "
+		<< targ->value->id << "; goto again;}";
+	return out;
+}
+
+std::ostream &ObjCGotoCodeGen::RET( bool inFinish )
+{
+	out << "{_cs = self->_st[--self->_top]; goto again;}";
+	return out;
+}
+
+std::ostream &ObjCGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
+{
+	out << "{self->_st[self->_top++] = _cs; _cs = (";
+	INLINE_LIST( ilItem->children, targState, inFinish );
+	out << "); goto again;}";
+	return out;
+}
+
+void ObjCGotoCodeGen::writeOutCode()
+{
+	out <<
+		"static int "; FSM_NAME() << "_start = "; START_STATE_ID() << ";\n"
+		"\n";
+
+	if ( anyActions() ) {
+		out <<
+			"static "; ARRAY_TYPE(maxActArrItem) << " "; FSM_NAME() << "_a[] = {\n";
+			ACTIONS_ARRAY() <<
+			"};\n"
+			"\n";
+	}
+
+	out <<
+		"@implementation "; FSM_NAME() << "\n\n"
+
+		"- (int)initFsm;\n"
+		"{\n"
+		"\tself->curs = "; FSM_NAME() << "_start;\n";
+
+	/* If there are any calls, then the stack top needs initialization. */
+	if ( anyActionCalls() || anyActionRets() )
+		out << "\tself->_top = 0;\n\n";
+
+	INIT_CODE() <<
+		"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n"
+		"}\n"
+		"\n";
+	
+	out <<
+		"- (int) executeWithData:("; EL_TYPE() << " *)_data len:(int)_len;\n"
+		"{\n"
+		"\t"; EL_TYPE() << " *_p = _data - 1;\n"
+		"\t"; EL_TYPE() << " *_pe = _data + _len;\n"
+		"\tint _cs = self->curs";
+
+	if ( anyRegCurStateRef() )
+		out << ", _ps = 0";
+	out << ";\n";
+
+	if ( anyActions() ) {
+		out << "\t";
+		ARRAY_TYPE(maxActArrItem) << " *_acts, _nacts;\n";
+	}
+
+	out << 
+		"\tself->curs = -1;\n"
+		"\n";
+
+	out << 
+		"again:\n"
+			"\tif ( ++_p == _pe )\n"
+				"\t\tgoto out;\n";
+
+	if ( anyEofActions() ) {
+		out <<
+			"\tif ( _p == 0 )\n"
+				"\t\tgoto eofActions;\n"
+			"\n";
+	}
+
+	out <<
+			"\tswitch ( _cs ) {\n";
+				STATE_GOTOS() <<
+			"\t}\n"
+		"\n";
+		TRANSITIONS() <<
+		"\n";
+
+	if ( anyActions() ) EXEC_FUNCS() << "\n";
+
+	if ( anyEofActions() ) {
+		out <<
+		"eofActions:\n"
+			"\t_p = 0; _pe = ( ("; EL_TYPE() << "*)0 ) + sizeof("; EL_TYPE() << ");\n"
+			"\tswitch( _cs ) {\n";
+				FINISH_CASES() << 
+			"\t}\n"
+		"\n";
+	}
+
+	out <<
+		"out:\n"
+			"\tself->curs = _cs;\n";
+
+	if ( redFsm->errState != 0 ) {
+		out <<
+			"\tif ( _cs == "; ERROR_STATE() << " ) return -1;\n";
+	}
+
+	out <<
+			"\treturn ( _cs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n"
+		"}\n"
+		"\n";
+	
+	out <<
+		"- (int)finish;\n"
+		"{\n";
+	
+	if ( anyEofActions() ) {
+		/* May need to execute code in the action loop. Must use special
+		 * params to execute func. */
+		out <<
+			"\treturn [self executeWithData:0 len:1];\n";
+	}
+	else {
+		// XXX this can be optimized to take out the double self->curs read in
+		// the case where there is an errState
+		if ( redFsm->errState != 0 ) {
+			out << 
+			"\tif ( self->curs == "; ERROR_STATE() << " ) return -1;\n";
+		}
+
+		out <<
+			"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n";
+	}
+
+	out <<
+		"}\n"
+		"\n";
+
+	for ( ContextMap::Iter ctx = parseData->contextMap; ctx.lte(); ctx++ ) {
+		/* Write out the vect. */
+		out << "unsigned char _"; FSM_NAME() << "_ctxdata_" << ctx->key << "[] = {\n\t";
+		CONTEXT( ctx->value->id );
+		out << "};\n\n";
+	}
+	
+	out <<
+		"@end\n\n";
+}
+
