@@ -57,14 +57,14 @@ std::ostream &IpGotoCodeGen::GOTO_HEADER( RedStateAp *state )
 
 			for ( ActionTable::Iter item = trans->action->key; item.lte(); item++ ) {
 				/* Get the function data. */
-				Action *action = parseData->actionIndex[item->value];
+				Action *act = parseData->actionIndex[item->value];
 
 				/* Write the preprocessor line info for going into the 
 				 * source file. */
-				out << "#line " << action->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
+				out << "#line " << act->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
 
 				/* Wrap the block in brakets. */
-				out << "\t{"; ACTION(action, trans->targ->id, false) << "}\n";
+				out << "\t{"; INLINE_LIST(act->inlineList, trans->targ->id, false) << "}\n";
 			}
 
 			/* If the action contains a next then we need to reload, otherwise
@@ -138,14 +138,14 @@ std::ostream &IpGotoCodeGen::FINISH_CASES()
 			out << "\n";
 			for ( ActionTable::Iter item = st->eofAction->key; item.lte(); item++ ) {
 				/* Get the action item. */
-				Action *action = parseData->actionIndex[item->value];
+				Action *act = parseData->actionIndex[item->value];
 				
 				/* Write the preprocessor line info for going into the 
 				 * source file. */
-				out << "#line " << action->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
+				out << "#line " << act->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
 
 				/* Wrap the block in brakets. */
-				out << "\t{"; ACTION(action, STATE_ERR_STATE, true) << "}\n";
+				out << "\t{"; INLINE_LIST(act->inlineList, STATE_ERR_STATE, true) << "}\n";
 			}
 			out << "\tbreak;\n";
 		}
@@ -159,6 +159,20 @@ std::ostream &IpGotoCodeGen::FINISH_CASES()
 	return out;
 }
 
+void IpGotoCodeGen::setLabelsNeeded( InlineList *inlineList )
+{
+	for ( InlineList::Iter item = *inlineList; item.lte(); item++ ) {
+		if ( item->type == InlineItem::Goto || item->type == InlineItem::Call ) {
+			/* Get the target and mark it as needing a label. */
+			RedEntryMapEl *targ = redFsm->entryMap.find( item->nameTarg->id );
+			targ->value->labelNeeded = true;
+		}
+
+		if ( item->children != 0 )
+			setLabelsNeeded( item->children );
+	}
+}
+
 /* Set up labelNeeded flag for each state. */
 void IpGotoCodeGen::setLabelsNeeded()
 {
@@ -178,20 +192,9 @@ void IpGotoCodeGen::setLabelsNeeded()
 		if ( trans->action != 0 ) {
 			/* Loop the actions. */
 			for ( ActionTable::Iter act = trans->action->key; act.lte(); act++ ) {
-				/* Get the action. */
+				/* Get the action and walk it's tree. */
 				Action *action = parseData->actionIndex[act->value];
-
-				/* Loop it's block items. */
-				for ( BlockIter item = action->data; item.lte(); item++ ) {
-					if ( item.type == CB_GOTO || item.type == CB_CALL ) {
-						/* Get the target. */
-						NameInst *name = action->nameTargs[item.name];
-						RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-
-						/* Need a label for this target. */
-						targ->value->labelNeeded = true;
-					}
-				}
+				setLabelsNeeded( action->inlineList );
 			}
 		}
 	}
@@ -209,6 +212,7 @@ std::ostream &IpGotoCodeGen::TARGS( bool inFinish, int targState )
 	return out;
 }
 
+
 /* Init base data. */
 CIpGotoCodeGen::CIpGotoCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
@@ -223,11 +227,11 @@ std::ostream &CIpGotoCodeGen::GOTO( NameInst *name, bool inFinish )
 	if ( !inFinish )
 		out << "{goto st" << targ->value->id << ";}";
 	else {
-		out << "{fsm->_cs = " << targ->value->id;
+		out << "{_cs = " << targ->value->id << "; ";
 		if ( anyEofActionHold() )
-			out << "; goto again;}";
+			out << "goto again;}";
 		else 
-			out << "; goto out;}";
+			out << "goto out;}";
 	}
 	return out;
 }
@@ -236,109 +240,68 @@ std::ostream &CIpGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish
 {
 	/* Lookup the target. Always guaranteed to return just one target. */
 	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	if ( !inFinish ) {
-		out << "{fsm->_st[fsm->_top++] = " << 
-				targState << "; goto st" << targ->value->id << ";}";
-	}
+	out << "{fsm->_st[fsm->_top++] = " << targState << "; ";
+	if ( !inFinish )
+		out << "goto st" << targ->value->id << ";}";
 	else {
-		out << "{fsm->_st[fsm->_top++] = " << 
-				targState << "; fsm->_cs = " << targ->value->id;
+		out << "_cs = " << targ->value->id << "; ";
 		if ( anyEofActionHold() )
-			out << "; goto again;}";
+			out << "goto again;}";
 		else 
-			out << "; goto out;}";
+			out << "goto out;}";
 	}
 	return out;
 }
 
 std::ostream &CIpGotoCodeGen::RET( bool inFinish )
 {
-	if ( !inFinish )
-		out << "{_cs = fsm->_st[--fsm->_top]; goto again;}";
-	else {
-		out << "{fsm->_cs = fsm->_st[--fsm->_top]";
-		if ( anyEofActionHold() )
-			out << "; goto again;}";
-		else 
-			out << "; goto out;}";
-	}
+	out << "{_cs = fsm->_st[--fsm->_top]; ";
+	if ( !inFinish || anyEofActionHold() )
+		out << "goto again;}";
+	else 
+		out << "goto out;}";
 	return out;
 }
 
-std::ostream &CIpGotoCodeGen::GOTOE( char *stateExpr, bool inFinish )
+std::ostream &CIpGotoCodeGen::GOTOE( InlineItem *ilItem, bool inFinish )
 {
-	if ( !inFinish ) {
-		out << "{_cs = (" << stateExpr << "); goto again;}";
-	}
-	else {
-		out << "{fsm->_cs = (" << stateExpr;
-		if ( anyEofActionHold() )
-			out << "); goto again;}";
-		else 
-			out << "); goto out;}";
-	}
+	out << "{_cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << "); ";
+	if ( !inFinish || anyEofActionHold() )
+		out << "goto again;}";
+	else 
+		out << "goto out;}";
 	return out;
 }
 
-std::ostream &CIpGotoCodeGen::CALLE( char *stateExpr, int targState, bool inFinish )
+std::ostream &CIpGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
 {
-	if ( !inFinish ) {
-		out << "{fsm->_st[fsm->_top++] = " << 
-				targState << "; _cs = (" << 
-				stateExpr << "); goto again;}";
-	}
-	else {
-		out << "{fsm->_st[fsm->_top++] = " << 
-				targState << "; fsm->_cs = (" << stateExpr;
-		if ( anyEofActionHold() )
-			out << "); goto again;}";
-		else 
-			out << "); goto out;}";
-	}
+	out << "{fsm->_st[fsm->_top++] = " << targState << "; _cs = (";
+	INLINE_LIST( ilItem->children, targState, inFinish );
+	out << "); ";
+	if ( !inFinish || anyEofActionHold() )
+		out << "goto again;}";
+	else 
+		out << "goto out;}";
 	return out;
 }
 
 std::ostream &CIpGotoCodeGen::NEXT( NameInst *name, bool inFinish )
 {
 	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	if ( ! inFinish )
-		out << "_cs = " << targ->value->id << ";";
-	else
-		out << "fsm->_cs = " << targ->value->id << ";";
+	out << "_cs = " << targ->value->id << ";";
 	return out;
 }
 
-std::ostream &CIpGotoCodeGen::NEXTE( char *stateExpr, bool inFinish )
+std::ostream &CIpGotoCodeGen::NEXTE( InlineItem *ilItem, bool inFinish )
 {
-	if ( !inFinish )
-		out << "_cs = (" << stateExpr << ");";
-	else
-		out << "fsm->_cs = (" << stateExpr << ");";
+	out << "_cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << ");";
 	return out;
 }
 
-void CIpGotoCodeGen::writeOutHeader()
-{
-	out <<
-		"/* Only non-static data: current state. */\n"
-		"struct "; FSM_NAME() << "\n"
-		"{\n"
-		"	int _cs;\n";
-		STRUCT_DATA() <<
-		"};\n"
-		"\n"
-		"/* Init the fsm. */\n"
-		"int "; FSM_NAME() << "_init( struct "; FSM_NAME() << " *fsm );\n"
-		"\n"
-		"/* Execute some chunk of data. */\n"
-		"int "; FSM_NAME() << "_execute( struct "; FSM_NAME() << " *fsm, "; EL_TYPE() << 
-				" *data, int len );\n"
-		"\n"
-		"/* Indicate to the fsm tha there is no more data. */\n"
-		"int "; FSM_NAME() << "_finish( struct "; FSM_NAME() << " *fsm );\n"
-		"\n";
-}
-
 void CIpGotoCodeGen::writeOutCode()
 {
 	out <<
@@ -346,14 +309,14 @@ void CIpGotoCodeGen::writeOutCode()
 		"\n"
 		"int "; FSM_NAME() << "_init( struct "; FSM_NAME() << " *fsm )\n"
 		"{\n"
-		"	fsm->_cs = "; FSM_NAME() << "_start;\n";
+		"	fsm->curs = "; FSM_NAME() << "_start;\n";
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
 		out << "	fsm->_top = 0;\n";
 
 	INIT_CODE() <<
-		"	if ( fsm->_cs >= "; FIRST_FINAL_STATE() << " )\n"
+		"	if ( fsm->curs >= "; FIRST_FINAL_STATE() << " )\n"
 		"		return 1;\n"
 		"	return 0;\n"
 		"}\n"
@@ -363,11 +326,15 @@ void CIpGotoCodeGen::writeOutCode()
 		"{\n"
 		"	"; EL_TYPE() << " *_p = _data-1;\n"
 		"	"; EL_TYPE() << " *_pe = _data+_len;\n"
-		"	int _cs = fsm->_cs";
+		"	int _cs = fsm->curs";
 
 	if ( anyRegCurStateRef() )
 		out << ", _ps";
-	out << ";\n\n";
+
+	out << 
+		";\n"
+		"	fsm->curs = -1;\n"
+		"\n";
 
 	/* Only needed by return statements. */
 	if ( anyRegActionRets() || anyRegActionByValControl() || anyRegNextStmt() )
@@ -384,7 +351,7 @@ void CIpGotoCodeGen::writeOutCode()
 	
 	out <<
 		"out:\n"
-		"	fsm->_cs = _cs;\n";
+		"	fsm->curs = _cs;\n";
 
 	if ( redFsm->errState != 0 ) {
 		out <<
@@ -414,6 +381,10 @@ void CIpGotoCodeGen::writeOutCode()
 		}
 	}
 
+	out << 
+		"	int _cs = fsm->curs;\n"
+		"	fsm->curs = -1;\n";
+
 	if ( anyEofActionHold() ) {
 		out <<
 			"again:\n"
@@ -422,7 +393,7 @@ void CIpGotoCodeGen::writeOutCode()
 	}
 
 	out <<
-		"	switch ( fsm->_cs ) {\n";
+		"	switch ( _cs ) {\n";
 		FINISH_CASES() <<
 		"	}\n"
 		"\n";
@@ -433,166 +404,134 @@ void CIpGotoCodeGen::writeOutCode()
 	if ( anyEofActionControl() || anyEofActionHold() )
 		out << "	out:\n";
 
+	out << "	fsm->curs = _cs;\n";
+
 	if ( redFsm->errState != 0 ) {
 		out <<
-			"	if ( fsm->_cs == "; ERROR_STATE() << " )\n"
+			"	if ( fsm->curs == "; ERROR_STATE() << " )\n"
 			"		return -1;\n";
 	}
 
 	out <<
-		"	if ( fsm->_cs >= "; FIRST_FINAL_STATE() << " )\n"
+		"	if ( fsm->curs >= "; FIRST_FINAL_STATE() << " )\n"
 		"		return 1;\n"
 		"	return 0;\n"
 		"}\n"
 		"\n";
+
+	for ( ContextMap::Iter ctx = parseData->contextMap; ctx.lte(); ctx++ ) {
+		/* Write out the vect. */
+		out << "unsigned char _"; FSM_NAME() << "_ctxdata_" << ctx->key << "[] = {\n\t";
+		CONTEXT( ctx->value->id );
+		out << "};\n\n";
+	}
 }
 
+
 /* Init base data. */
-CCIpGotoCodeGen::CCIpGotoCodeGen( char *fsmName, ParseData *parseData, 
+CppIpGotoCodeGen::CppIpGotoCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 :
 	IpGotoCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &CCIpGotoCodeGen::GOTO( NameInst *name, bool inFinish )
+std::ostream &CppIpGotoCodeGen::GOTO( NameInst *name, bool inFinish )
 {
 	/* Lookup the target. Always guaranteed to return just one target. */
 	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
 	if ( !inFinish )
 		out << "{goto st" << targ->value->id << ";}";
 	else {
-		out << "{this->_cs = " << targ->value->id;
+		out << "{_cs = " << targ->value->id << "; ";
 		if ( anyEofActionHold() )
-			out << "; goto again;}";
+			out << "goto again;}";
 		else 
-			out << "; goto out;}";
+			out << "goto out;}";
 	}
 	return out;
 }
 
-std::ostream &CCIpGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
+std::ostream &CppIpGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
 {
 	/* Lookup the target. Always guaranteed to return just one target. */
 	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	if ( !inFinish ) {
-		out << "{this->_st[this->_top++] = " << 
-				targState << "; goto st" << targ->value->id << ";}";
-	}
-	else {
-		out << "{this->_st[this->_top++] = " << 
-				targState << "; this->_cs = " << targ->value->id;
-		if ( anyEofActionHold() )
-			out << "; goto again;}";
-		else 
-			out << "; goto out;}";
-	}
-	return out;
-}
-
-std::ostream &CCIpGotoCodeGen::RET( bool inFinish )
-{
+	out << "{this->_st[this->_top++] = " << targState << "; ";
 	if ( !inFinish )
-		out << "{_cs = this->_st[--this->_top]; goto again;}";
+		out << "goto st" << targ->value->id << ";}";
 	else {
-		out << "{this->_cs = this->_st[--this->_top]";
+		out << "_cs = " << targ->value->id << "; ";
 		if ( anyEofActionHold() )
-			out << "; goto again;}";
+			out << "goto again;}";
 		else 
-			out << "; goto out;}";
+			out << "goto out;}";
 	}
 	return out;
 }
 
-std::ostream &CCIpGotoCodeGen::GOTOE( char *stateExpr, bool inFinish )
+std::ostream &CppIpGotoCodeGen::RET( bool inFinish )
 {
-	/* Lookup the target. Always guaranteed to return just one target. */
-	if ( !inFinish ) {
-		out << "{_cs = (" << stateExpr << "); goto again;}";
-	}
-	else {
-		out << "{this->_cs = (" << stateExpr;
-		if ( anyEofActionHold() )
-			out << "); goto again;}";
-		else 
-			out << "); goto out;}";
-	}
+	out << "{_cs = this->_st[--this->_top]; ";
+	if ( !inFinish || anyEofActionHold() )
+		out << "goto again;}";
+	else 
+		out << "goto out;}";
 	return out;
 }
 
-std::ostream &CCIpGotoCodeGen::CALLE( char *stateExpr, int targState, bool inFinish )
+std::ostream &CppIpGotoCodeGen::GOTOE( InlineItem *ilItem, bool inFinish )
 {
-	if ( !inFinish ) {
-		out << "{this->_st[this->_top++] = " << 
-				targState << "; _cs = (" << 
-				stateExpr << "); goto again;}";
-	}
-	else {
-		out << "{this->_st[this->_top++] = " << 
-				targState << "; this->_cs = (" << stateExpr;
-		if ( anyEofActionHold() )
-			out << "); goto again;}";
-		else 
-			out << "); goto out;}";
-	}
+	out << "{_cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << "); ";
+	if ( !inFinish || anyEofActionHold() )
+		out << "goto again;}";
+	else 
+		out << "goto out;}";
 	return out;
 }
 
-std::ostream &CCIpGotoCodeGen::NEXT( NameInst *name, bool inFinish )
+std::ostream &CppIpGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
+{
+	out << "{this->_st[this->_top++] = " << targState << "; _cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << "); ";
+	if ( !inFinish || anyEofActionHold() )
+		out << "); goto again;}";
+	else 
+		out << "); goto out;}";
+	return out;
+}
+
+std::ostream &CppIpGotoCodeGen::NEXT( NameInst *name, bool inFinish )
 {
 	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	if ( ! inFinish )
-		out << "_cs = " << targ->value->id << ";";
-	else
-		out << "this->_cs = " << targ->value->id << ";";
+	out << "_cs = " << targ->value->id << ";";
 	return out;
 }
 
-std::ostream &CCIpGotoCodeGen::NEXTE( char *stateExpr, bool inFinish )
+std::ostream &CppIpGotoCodeGen::NEXTE( InlineItem *ilItem, bool inFinish )
 {
-	if ( !inFinish )
-		out << "_cs = (" << stateExpr << ");";
-	else
-		out << "this->_cs = (" << stateExpr << ");";
+	out << "_cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << ");";
 	return out;
 }
 
-void CCIpGotoCodeGen::writeOutHeader()
-{
-	out <<
-		"/* Only non-static data: current state. */\n"
-		"class "; FSM_NAME() << "\n"
-		"{\n"
-		"public:\n"
-		"	int _cs;\n"
-		"\n"
-		"	/* Init the fsm. */\n"
-		"	int init( );\n"
-		"\n"
-		"	/* Execute some chunk of data. */\n"
-		"	int execute( "; EL_TYPE() << " *data, int len );\n"
-		"\n"
-		"	/* Indicate to the fsm tha there is no more data. */\n"
-		"	int finish( );\n";
-		STRUCT_DATA() <<
-		"};\n"
-		"\n";
-}
-
-void CCIpGotoCodeGen::writeOutCode()
+void CppIpGotoCodeGen::writeOutCode()
 {
 	out <<
 		"static int "; FSM_NAME() << "_start = "; START_STATE_ID() << ";\n"
 		"\n"
 		"int "; FSM_NAME() << "::init( )\n"
 		"{\n"
-		"	this->_cs = "; FSM_NAME() << "_start;\n";
+		"	this->curs = "; FSM_NAME() << "_start;\n";
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
 		out << "	this->_top = 0;\n";
 
 	INIT_CODE() <<
-		"	if ( this->_cs >= "; FIRST_FINAL_STATE() << " )\n"
+		"	if ( this->curs >= "; FIRST_FINAL_STATE() << " )\n"
 		"		return 1;\n"
 		"	return 0;\n"
 		"}\n"
@@ -601,11 +540,15 @@ void CCIpGotoCodeGen::writeOutCode()
 		"{\n"
 		"	"; EL_TYPE() << " *_p = _data-1;\n"
 		"	"; EL_TYPE() << " *_pe = _data+_len;\n"
-		"	int _cs = this->_cs";
+		"	int _cs = this->curs";
 
 	if ( anyRegCurStateRef() )
 		out << ", _ps";
-	out << ";\n\n";
+
+	out << 
+		";\n"
+		"	this->curs = -1;\n"
+		"\n";
 
 	/* Needed by returns, by-value control and next statements. */
 	if ( anyRegActionRets() || anyRegActionByValControl() || anyRegNextStmt() )
@@ -622,7 +565,7 @@ void CCIpGotoCodeGen::writeOutCode()
 
 	out <<
 		"out:\n"
-		"	this->_cs = _cs;\n";
+		"	this->curs = _cs;\n";
 
 	if ( redFsm->errState != 0 ) {
 		out <<
@@ -652,6 +595,11 @@ void CCIpGotoCodeGen::writeOutCode()
 		}
 	}
 
+	out << 
+		"	int _cs = this->curs;\n"
+		"	this->curs = -1;\n";
+
+
 	if ( anyEofActionHold() ) {
 		out <<
 			"again:\n"
@@ -660,7 +608,7 @@ void CCIpGotoCodeGen::writeOutCode()
 	}
 
 	out <<
-		"	switch ( this->_cs ) {\n";
+		"	switch ( _cs ) {\n";
 		FINISH_CASES() <<
 		"	}\n"
 		"\n";
@@ -672,16 +620,242 @@ void CCIpGotoCodeGen::writeOutCode()
 	if ( anyEofActionControl() || anyEofActionHold() )
 		out << "	out:\n";
 
+	out << "	this->curs = _cs;\n";
 	if ( redFsm->errState != 0 ) {
 		out <<
-			"	if ( this->_cs == "; ERROR_STATE() << " )\n"
+			"	if ( this->curs == "; ERROR_STATE() << " )\n"
 			"		return -1;\n";
 	}
 
 	out <<
-		"	if ( this->_cs >= "; FIRST_FINAL_STATE() << " )\n"
+		"	if ( this->curs >= "; FIRST_FINAL_STATE() << " )\n"
 		"		return 1;\n"
 		"	return 0;\n"
 		"}\n"
 		"\n";
+
+	for ( ContextMap::Iter ctx = parseData->contextMap; ctx.lte(); ctx++ ) {
+		out << "unsigned char "; FSM_NAME() << "::_ctxdata_" << ctx->key << "[] = {\n\t";
+		CONTEXT( ctx->value->id );
+		out << "};\n\n";
+	}
+}
+
+
+/* Init base data. */
+ObjCIpGotoCodeGen::ObjCIpGotoCodeGen( char *fsmName, ParseData *parseData, 
+		RedFsmAp *redFsm, std::ostream &out )
+:
+	IpGotoCodeGen(fsmName, parseData, redFsm, out)
+{ }
+
+std::ostream &ObjCIpGotoCodeGen::GOTO( NameInst *name, bool inFinish )
+{
+	/* Lookup the target. Always guaranteed to return just one target. */
+	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
+	if ( !inFinish )
+		out << "{goto st" << targ->value->id << ";}";
+	else {
+		out << "{_cs = " << targ->value->id << "; ";
+		if ( anyEofActionHold() )
+			out << "goto again;}";
+		else 
+			out << "goto out;}";
+	}
+	return out;
+}
+
+std::ostream &ObjCIpGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
+{
+	/* Lookup the target. Always guaranteed to return just one target. */
+	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
+	out << "{self->_st[self->_top++] = " << targState << "; ";
+	if ( !inFinish )
+		out << "goto st" << targ->value->id << ";}";
+	else {
+		out << "_cs = " << targ->value->id << "; ";
+		if ( anyEofActionHold() )
+			out << "goto again;}";
+		else 
+			out << "goto out;}";
+	}
+	return out;
+}
+
+std::ostream &ObjCIpGotoCodeGen::RET( bool inFinish )
+{
+	out << "{_cs = self->_st[--self->_top]; ";
+	if ( !inFinish || anyEofActionHold() )
+		out << "goto again;}";
+	else 
+		out << "goto out;}";
+	return out;
+}
+
+std::ostream &ObjCIpGotoCodeGen::GOTOE( InlineItem *ilItem, bool inFinish )
+{
+	out << "{_cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << "); ";
+	if ( !inFinish || anyEofActionHold() )
+		out << "goto again;}";
+	else 
+		out << "goto out;}";
+	return out;
+}
+
+std::ostream &ObjCIpGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
+{
+	out << "{self->_st[self->_top++] = " << targState << "; _cs = (";
+	INLINE_LIST( ilItem->children, targState, inFinish );
+	out << "); ";
+	if ( !inFinish || anyEofActionHold() )
+		out << "); goto again;}";
+	else 
+		out << "); goto out;}";
+	return out;
+}
+
+std::ostream &ObjCIpGotoCodeGen::NEXT( NameInst *name, bool inFinish )
+{
+	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
+	out << "_cs = " << targ->value->id << ";";
+	return out;
+}
+
+std::ostream &ObjCIpGotoCodeGen::NEXTE( InlineItem *ilItem, bool inFinish )
+{
+	out << "_cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << ");";
+	return out;
+}
+
+void ObjCIpGotoCodeGen::writeOutCode()
+{
+	out <<
+		"static int "; FSM_NAME() << "_start = "; START_STATE_ID() << ";\n"
+		"\n";
+		
+	out <<
+		"@implementation "; FSM_NAME() << "\n\n"
+
+		"- (int)initFsm;\n"
+		"{\n"
+		"\tself->curs = "; FSM_NAME() << "_start;\n";
+
+	/* If there are any calls, then the stack top needs initialization. */
+	if ( anyActionCalls() || anyActionRets() )
+		out << "\tself->_top = 0;\n";
+
+	INIT_CODE() <<
+			"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n"
+		"}\n"
+		"\n";
+		
+	out <<
+		"- (int) executeWithData:("; EL_TYPE() << " *)_data len:(int)_len;\n"
+		"{\n"
+		"\t"; EL_TYPE() << " *_p = _data - 1;\n"
+		"\t"; EL_TYPE() << " *_pe = _data + _len;\n"
+		"\tint _cs = self->curs";
+
+	if ( anyRegCurStateRef() )
+		out << ", _ps";
+
+	out << ";\n";
+		
+	out <<
+		"\tself->curs = -1;\n"
+		"\n";
+
+	/* Needed by returns, by-value control and next statements. */
+	if ( anyRegActionRets() || anyRegActionByValControl() || anyRegNextStmt() )
+		out << "again:\n";
+
+	out <<
+		"\tif ( ++_p == _pe )\n"
+			"\t\tgoto out;\n"
+		"\tswitch ( _cs ) {\n";
+		STATE_GOTOS() <<
+		"\t}\n";
+		EXIT_STATES() << 
+		"\n";
+
+	out <<
+		"out:\n"
+			"\tself->curs = _cs;\n";
+
+	if ( redFsm->errState != 0 ) {
+		out <<
+			"\tif ( _cs == "; ERROR_STATE() << " ) return -1;\n";
+	}
+
+	out <<
+			"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n"
+		"}\n"
+		"\n";
+	
+	out <<
+		"- (int)finish;\n"
+		"{\n";
+	
+	if ( anyEofActionCharRef() ) {
+		out << "	"; 
+		if ( ! anyEofActionHold() ) {
+			EL_TYPE() << " *_p = 0;\n";
+		}
+		else {
+			EL_TYPE() << " *_p = (("; EL_TYPE() << "*)0)-1" << 
+					", *_pe = (("; EL_TYPE() << "*)0)+1;\n";
+		}
+	}
+
+	out << 
+		"\tint _cs = self->curs;\n"
+		"\tself->curs = -1;\n";
+
+
+	if ( anyEofActionHold() ) {
+		out <<
+		"again:\n"
+			"\tif ( ++_p == _pe )\n"
+				"\t\tgoto out;\n";
+	}
+
+	out <<
+			"\tswitch ( _cs ) {\n";
+		FINISH_CASES() <<
+			"\t}\n"
+		"\n";
+
+	if ( anyEofActionHold() ) {
+		out << "\tgoto again;\n";
+	}
+	
+	if ( anyEofActionControl() || anyEofActionHold() )
+		out << "out:\n";
+
+	out << "\tself->curs = _cs;\n";
+	if ( redFsm->errState != 0 ) {
+		out <<
+			"\tif ( self->curs == "; ERROR_STATE() << " ) return -1;\n";
+	}
+
+	out <<
+		"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n";
+
+	out <<
+		"}\n"
+		"\n";
+
+	for ( ContextMap::Iter ctx = parseData->contextMap; ctx.lte(); ctx++ ) {
+		/* Write out the vect. */
+		out << "unsigned char _"; FSM_NAME() << "_ctxdata_" << ctx->key << "[] = {\n\t";
+		CONTEXT( ctx->value->id );
+		out << "};\n\n";
+	}
+
+	out <<
+		"@end\n\n";
 }
