@@ -22,7 +22,6 @@
 #include <iostream>
 #include <iomanip>
 #include <errno.h>
-#include <limits.h>
 #include <stdlib.h>
 
 /* Parsing. */
@@ -38,6 +37,8 @@
 /* Code generators. */
 #include "tabcodegen.h"
 #include "ftabcodegen.h"
+#include "flatcodegen.h"
+#include "fflatcodegen.h"
 #include "gotocodegen.h"
 #include "fgotocodegen.h"
 #include "ipgotocodegen.h"
@@ -51,29 +52,6 @@ using namespace std;
 
 char machineMain[] = "main";
 
-/* The minimum chars are the absolute value of the real minimum because the
- * sign is stored separately in integers read in so they are compared in the
- * positive. Each is casted to an unsigned because the data part of the number
- * is in unsigned int size. 
- */
-#define RL_CHAR_MIN    ((long)((char)CHAR_MIN))
-#define RL_CHAR_MAX    ((long)((char)CHAR_MAX))
-#define RL_UCHAR_MIN   ((unsigned long)((unsigned char)0))
-#define RL_UCHAR_MAX   ((unsigned long)((unsigned char)UCHAR_MAX))
-#define RL_SHORT_MIN   ((long)((short)SHRT_MIN))
-#define RL_SHORT_MAX   ((long)((short)SHRT_MAX))
-#define RL_USHORT_MIN  ((unsigned long)((unsigned short)0))
-#define RL_USHORT_MAX  ((unsigned long)((unsigned short)USHRT_MAX))
-#define RL_INT_MIN     ((long)((int)INT_MIN))
-#define RL_INT_MAX     ((long)((int)INT_MAX))
-#define RL_UINT_MIN    ((unsigned long)((unsigned int)0))
-#define RL_UINT_MAX    ((unsigned long)((unsigned int)UINT_MAX))
-#define RL_LONG_MIN    ((long)LONG_MIN)
-#define RL_LONG_MAX    ((long)LONG_MAX)
-#define RL_ULONG_MIN   ((unsigned long)0)
-#define RL_ULONG_MAX   ((unsigned long)LONG_MAX)
-
-
 InputLoc::InputLoc( const BISON_YYLTYPE &loc )
 :
 	line(loc.first_line), 
@@ -82,12 +60,12 @@ InputLoc::InputLoc( const BISON_YYLTYPE &loc )
 }
 
 Action::Action( const InputLoc &loc, char *name, 
-		char *data, const NameRefList &nameRefs )
+		InlineList *inlineList, const NameRefList &nameRefs )
 :
 	loc(loc),
 	name(name),
-	data(data), 
-	id(0),
+	inlineList(inlineList), 
+	actionId(0),
 	numTransRefs(0),
 	numEofRefs(0),
 	nameRefs(nameRefs)
@@ -97,43 +75,8 @@ Action::Action( const InputLoc &loc, char *name,
 Action::~Action()
 {
 	delete[] name;
-	delete[] data;
 }
 
-/* Initialization. */
-BlockIter::BlockIter( char *bd )
-{
-	type = *bd;
-	data = bd + 1;
-	name = 0;
-}
-
-void BlockIter::operator=( char *bd )
-{
-	type = *bd;
-	data = bd + 1;
-	name = 0;
-}
-
-void BlockIter::operator++(int)
-{
-	/* If there is a data argument, skip over it. 
-	 * If there is a name, skip over it. */
-	switch ( type ) {
-		case CB_TEXT: case CB_STACK:
-		case CB_GOTOE: case CB_NEXTE:
-		case CB_CALLE:
-			data += strlen(data) + 1;
-			break;
-		case CB_STATE: case CB_GOTO:
-		case CB_NEXT: case CB_CALL:
-			name += 1;
-			break;
-	}
-	/* Get the type and load the data. */
-	type = *data++;
-}
-		
 /* Perform minimization after an operation according 
  * to the command line args. */
 void afterOpMinimize( FsmAp *fsm )
@@ -630,6 +573,21 @@ FsmAp *makeBuiltin( BuiltinMachine builtin, ParseData *pd )
 	return retFsm;
 }
 
+/* Check if this name inst or any name inst below is referenced. */
+bool NameInst::anyRefsRec()
+{
+	if ( numRefs > 0 )
+		return true;
+
+	/* Recurse on children until true. */
+	for ( NameVect::Iter ch = childVect; ch.lte(); ch++ ) {
+		if ( (*ch)->anyRefsRec() )
+			return true;
+	}
+
+	return false;
+}
+
 /*
  * ParseData
  */
@@ -639,7 +597,7 @@ FsmAp *makeBuiltin( BuiltinMachine builtin, ParseData *pd )
 ParseData::ParseData( const String &fileName )
 :	
 	actionIndex(0),
-
+	baseClause(0),
 	nextPriorKey(0),
 	/* 0 is reserved for global error actions. */
 	nextLocalErrKey(1),
@@ -649,6 +607,7 @@ ParseData::ParseData( const String &fileName )
 	alphType(AT_Char),
 	elementType(0),
 	getKeyExpr(0),
+	nextContextId(0),
 	fileName(fileName),
 	errorCount(0),
 
@@ -695,6 +654,11 @@ void ParseData::initNameWalk()
 	curNameChild = 0;
 }
 
+/* Goes into the next child scope. The number of the child is already set up.
+ * We need this for the syncronous name tree and parse tree walk to work
+ * properly. It is reset on entry into a scope and advanced on poping of a
+ * scope. A call to enterNameScope should be accompanied by a corresponding
+ * popNameScope. */
 NameFrame ParseData::enterNameScope( bool isLocal, int numScopes )
 {
 	/* Save off the current data. */
@@ -715,6 +679,9 @@ NameFrame ParseData::enterNameScope( bool isLocal, int numScopes )
 	return retFrame;
 }
 
+/* Return from a child scope to a parent. The parent info must be specified as
+ * an argument and is obtained from the corresponding call to enterNameScope.
+ * */
 void ParseData::popNameScope( const NameFrame &frame )
 {
 	/* Pop the name scope. */
@@ -747,13 +714,13 @@ void ParseData::unsetObsoleteEntries( FsmAp *graph )
 	}
 }
 
-NameVect ParseData::resolvePart( NameInst *refFrom, char *data, bool recLabelsOnly )
+NameSet ParseData::resolvePart( NameInst *refFrom, char *data, bool recLabelsOnly )
 {
 	/* Queue needed for breadth-first search, load it with the start node. */
 	NameInstList nameQueue;
 	nameQueue.append( refFrom );
 
-	NameVect result;
+	NameSet result;
 	while ( nameQueue.length() > 0 ) {
 		/* Pull the next from location off the queue. */
 		NameInst *from = nameQueue.detachFirst();
@@ -763,7 +730,7 @@ NameVect ParseData::resolvePart( NameInst *refFrom, char *data, bool recLabelsOn
 		if ( from->children.findMulti( data, low, high ) ) {
 			/* Record all instances of the name. */
 			for ( ; low <= high; low++ )
-				result.append( low->value );
+				result.insert( low->value );
 		}
 
 		/* Name not there, do breadth-first operation of appending all
@@ -778,23 +745,23 @@ NameVect ParseData::resolvePart( NameInst *refFrom, char *data, bool recLabelsOn
 	return result;
 }
 
-void ParseData::resolveFrom( NameVect &result, NameInst *refFrom, 
+void ParseData::resolveFrom( NameSet &result, NameInst *refFrom, 
 		const NameRef &nameRef, int namePos )
 {
 	/* Look for the name in the owning scope of the factor with aug. */
-	NameVect partResult = resolvePart( refFrom, nameRef[namePos], false );
+	NameSet partResult = resolvePart( refFrom, nameRef[namePos], false );
 	
 	/* If there are more parts to the name then continue on. */
 	if ( ++namePos < nameRef.length() ) {
 		/* There are more components to the name, search using all the part
 		 * results as the base. */
-		for ( NameVect::Iter name = partResult; name.lte(); name++ )
+		for ( NameSet::Iter name = partResult; name.lte(); name++ )
 			resolveFrom( result, *name, nameRef, namePos );
 	}
 	else {
 		/* This is the last component, append the part results to the final
 		 * results. */
-		result.append( partResult );
+		result.insert( partResult );
 	}
 }
 
@@ -856,11 +823,11 @@ struct CmpNameInstLoc
 	}
 };
 
-void errorStateLabels( const NameVect &resolved )
+void errorStateLabels( const NameSet &resolved )
 {
 	MergeSort<NameInst*, CmpNameInstLoc> mergeSort;
 	mergeSort.sort( resolved.data, resolved.length() );
-	for ( NameVect::Iter res = resolved; res.lte(); res++ )
+	for ( NameSet::Iter res = resolved; res.lte(); res++ )
 		error((*res)->loc) << "  -> " << **res << endl;
 }
 
@@ -875,7 +842,7 @@ NameInst *ParseData::resolveStateRef( const NameRef &nameRef, InputLoc &loc, Act
 		/* If the action is referenced, resolve all of them. */
 		if ( action != 0 && action->actionRefs.length() > 0 ) {
 			/* Look for the name in all referencing scopes. */
-			NameVect resolved;
+			NameSet resolved;
 			for ( ActionRefs::Iter actRef = action->actionRefs; actRef.lte(); actRef++ )
 				resolveFrom( resolved, *actRef, nameRef, 0 );
 
@@ -885,7 +852,7 @@ NameInst *ParseData::resolveStateRef( const NameRef &nameRef, InputLoc &loc, Act
 				if ( resolved.length() > 1 ) {
 					/* Complain about the multiple references. */
 					error(loc) << "state reference " << nameRef << 
-							" resolves to multiple states" << endl;
+							" resolves to multiple entry points" << endl;
 					errorStateLabels( resolved );
 				}
 			}
@@ -894,7 +861,7 @@ NameInst *ParseData::resolveStateRef( const NameRef &nameRef, InputLoc &loc, Act
 
 	/* If not found in the local scope, look in global. */
 	if ( nameInst == 0 ) {
-		NameVect resolved;
+		NameSet resolved;
 		int fromPos = nameRef[0] != 0 ? 0 : 1;
 		resolveFrom( resolved, rootName, nameRef, fromPos );
 
@@ -904,58 +871,55 @@ NameInst *ParseData::resolveStateRef( const NameRef &nameRef, InputLoc &loc, Act
 			if ( resolved.length() > 1 ) {
 				/* Complain about the multiple references. */
 				error(loc) << "state reference " << nameRef << 
-						" resolves to multiple states" << endl;
+						" resolves to multiple entry points" << endl;
 				errorStateLabels( resolved );
 			}
 		}
 	}
 
-	if ( nameInst != 0 ) {
-		/* Note the reference in the name. This will cause the entry point to
-		 * survive to the end of the graph generating walk. */
-		nameInst->numRefs += 1;
-	}
-	else {
+	if ( nameInst == 0 ) {
 		/* If not found then complain. */
 		error(loc) << "could not resolve state reference " << nameRef << endl;
 	}
 	return nameInst;
 }
 
-void ParseData::resolveInitNameRefs()
+void ParseData::resolveNameRefs( InlineList *inlineList, Action *action )
 {
-	for ( InlineBlockList::Iter init = initCodeList; init.lte(); init++ ) {
-		/* Resolve any references to names. */
-		for ( BlockIter item = init->data; item.lte(); item++ ) {
-			switch ( item.type ) {
-			case CB_STATE:
-				NameInst *target = resolveStateRef( *init->nameRefs[item.name], 
-						init->loc, 0 );
-				init->nameTargs.append( target );
+	for ( InlineList::Iter item = *inlineList; item.lte(); item++ ) {
+		switch ( item->type ) {
+			case InlineItem::Entry: case InlineItem::Goto:
+			case InlineItem::Call: case InlineItem::Next: {
+				/* Resolve, pass action for local search. */
+				NameInst *target = resolveStateRef( *item->nameRef, item->loc, action );
+
+				/* Note the reference in the name. This will cause the entry
+				 * point to survive to the end of the graph generating walk. */
+				if ( target != 0 )
+					target->numRefs += 1;
+				item->nameTarg = target;
 				break;
 			}
+			default:
+				break;
 		}
 	}
 }
 
+void ParseData::resolveInitNameRefs()
+{
+	/* Resolve references in all init blocks. */
+	for ( InlineBlockList::Iter init = initCodeList; init.lte(); init++ )
+		resolveNameRefs( init->inlineList, 0 );
+}
+
 /* Resolve references to labels in actions. */
 void ParseData::resolveActionNameRefs()
 {
 	for ( ActionList::Iter act = actionList; act.lte(); act++ ) {
 		/* Only care about the actions that are referenced. */
-		if ( act->actionRefs.length() > 0 ) {
-			/* Resolve any references to names. */
-			for ( BlockIter item = act->data; item.lte(); item++ ) {
-				switch ( item.type ) {
-				case CB_STATE: case CB_GOTO: 
-				case CB_CALL: case CB_NEXT:
-					NameInst *target = resolveStateRef( *act->nameRefs[item.name], 
-							act->loc, act );
-					act->nameTargs.append( target );
-					break;
-				}
-			}
-		}
+		if ( act->actionRefs.length() > 0 )
+			resolveNameRefs( act->inlineList, act );
 	}
 }
 
@@ -1133,7 +1097,7 @@ void ParseData::fillActionIndex()
 	for ( ActionList::Iter act = actionList; act.lte(); act++ ) {
 		/* Only ever interested in referenced actions. */
 		if ( act->actionRefs.length() > 0 ) {
-			act->id = nextActionId;
+			act->actionId = nextActionId;
 			actionIndex[nextActionId] = act;
 			nextActionId += 1;
 		}
@@ -1276,8 +1240,13 @@ FsmAp *ParseData::makeAll()
 		else {
 			/* Check to see if the instance is ever referenced. */
 			NameInst *nameInst = nextNameScope();
-			if ( nameInst->numRefs > 0 )
+			if ( nameInst->anyRefsRec() )
 				graphs[numOthers++] = makeInstance( glel );
+			else {
+				/* Need to walk over the name tree item. */
+				NameFrame nameFrame = enterNameScope( true, 1 );
+				popNameScope( nameFrame );
+			}
 		}
 	}
 
@@ -1440,6 +1409,12 @@ FsmCodeGen *ParseData::makeCodeGen( RedFsmAp *redFsm )
 		case GenFTables:
 			codeGen = new CFTabCodeGen( fsmName, this, redFsm, *outStream );
 			break;
+		case GenFlat:
+			codeGen = new CFlatCodeGen( fsmName, this, redFsm, *outStream );
+			break;
+		case GenFFlat:
+			codeGen = new CFFlatCodeGen( fsmName, this, redFsm, *outStream );
+			break;
 		case GenGoto:
 			codeGen = new CGotoCodeGen( fsmName, this, redFsm, *outStream );
 			break;
@@ -1454,22 +1429,54 @@ FsmCodeGen *ParseData::makeCodeGen( RedFsmAp *redFsm )
 	case OutCppCode:
 		switch ( codeStyle ) {
 		case GenTables:
-			codeGen = new CCTabCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CppTabCodeGen( fsmName, this, redFsm, *outStream );
 			break;
 		case GenFTables:
-			codeGen = new CCFTabCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CppFTabCodeGen( fsmName, this, redFsm, *outStream );
+			break;
+		case GenFlat:
+			codeGen = new CppFlatCodeGen( fsmName, this, redFsm, *outStream );
+			break;
+		case GenFFlat:
+			codeGen = new CppFFlatCodeGen( fsmName, this, redFsm, *outStream );
 			break;
 		case GenGoto:
-			codeGen = new CCGotoCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CppGotoCodeGen( fsmName, this, redFsm, *outStream );
 			break;
 		case GenFGoto:
-			codeGen = new CCFGotoCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CppFGotoCodeGen( fsmName, this, redFsm, *outStream );
 			break;
 		case GenIpGoto:
-			codeGen = new CCIpGotoCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CppIpGotoCodeGen( fsmName, this, redFsm, *outStream );
 			break;
 		}
 		break;
+	case OutObjCCode:
+		switch ( codeStyle ) {
+		case GenTables:
+			codeGen = new ObjCTabCodeGen( fsmName, this, redFsm, *outStream );
+			break;
+		case GenFTables:
+			codeGen = new ObjCFTabCodeGen( fsmName, this, redFsm, *outStream );
+			break;
+		case GenFlat:
+			codeGen = new ObjCFlatCodeGen( fsmName, this, redFsm, *outStream );
+			break;
+		case GenFFlat:
+			codeGen = new ObjCFFlatCodeGen( fsmName, this, redFsm, *outStream );
+			break;
+		case GenGoto:
+			codeGen = new ObjCGotoCodeGen( fsmName, this, redFsm, *outStream );
+			break;
+		case GenFGoto:
+			codeGen = new ObjCFGotoCodeGen( fsmName, this, redFsm, *outStream );
+			break;
+		case GenIpGoto:
+			codeGen = new ObjCIpGotoCodeGen( fsmName, this, redFsm, *outStream );
+			break;
+		}
+		break;
+
 
 	/* Called codegen when generating something else. */
 	default:
@@ -1523,8 +1530,12 @@ void ParseData::generateCode( )
 		}
 		else {
 			/* For table driven machines the location of the state is used to
-			 * identify it so the states must be sorted by their final ids. */
+			 * identify it so the states must be sorted by their final ids.
+			 * Though having a deterministic ordering is important,
+			 * specifically preserving the depth first ordering is not because
+			 * states are stored in tables. */
 			redFsm->sortStatesByFinal();
+			redFsm->sequentialStateIds();
 		}
 
 		/* Find the first final state. This is the final state with the lowest
@@ -1533,7 +1544,12 @@ void ParseData::generateCode( )
 
 		/* Choose default transitions and the single transition. */
 		redFsm->chooseDefaultSpan();
-		redFsm->chooseSingle();
+		
+		/* Maybe do flat expand, otherwise choose single. */
+		if ( codeStyle == GenFlat || codeStyle == GenFFlat )
+			redFsm->makeFlat();
+		else
+			redFsm->chooseSingle();
 	}
 
 	/* If any errors have occured in the input file then don't write anything. */
