@@ -77,7 +77,7 @@ FsmAp *VarDef::walk( ParseData *pd )
 	 * the graph. */
 	if ( pd->curNameInst->numRefs > 0 )
 		rtnVal->setEntry( pd->curNameInst->id, rtnVal->startState );
-
+	
 	/* Pop the name scope. */
 	pd->popNameScope( nameFrame );
 	return rtnVal;
@@ -150,16 +150,22 @@ FsmAp *Join::walkJoin( ParseData *pd )
 	NameInst *startName = pd->curNameInst->start;
 	NameInst *finalName = pd->curNameInst->final;
 
-	int startId = 0;
+	int startId = -1;
 	if ( startName != 0 ) {
 		/* Take note that there was an implicit link to the start machine. */
 		pd->localNameScope->referencedNames.append( startName );
 		startId = startName->id;
 	}
 
+	/* A final id of -1 indicates there is no epsilon that references the
+	 * final state, therefor do not create one or set an entry point to it. */
+	int finalId = -1;
+	if ( finalName->numRefs > 0 )
+		finalId = finalName->id;
+
 	/* Join machines 1 and up onto machine 0. */
 	FsmAp *retFsm = fsms[0];
-	retFsm->joinOp( startId, finalName->id, fsms+1, exprList.length()-1 );
+	retFsm->joinOp( startId, finalId, fsms+1, exprList.length()-1 );
 
 	/* We can now unset entry points that are not longer used. */
 	pd->unsetObsoleteEntries( retFsm );
@@ -204,7 +210,7 @@ void Join::resolveNameRefs( ParseData *pd )
 		NameFrame nameFrame = pd->enterNameScope( true, 1 );
 
 		/* The join scope must contain a start label. */
-		NameVect resolved = pd->resolvePart( pd->localNameScope, "start", true );
+		NameSet resolved = pd->resolvePart( pd->localNameScope, "start", true );
 		if ( resolved.length() > 0 ) {
 			/* Take the first. */
 			pd->curNameInst->start = resolved[0];
@@ -420,72 +426,76 @@ void FactorWithAug::assignActions( ParseData *pd, FsmAp *graph, int *actionOrd )
 	for ( int i = 0; i < actions.length(); i++ )  {
 		switch ( actions[i].type ) {
 		case at_start:
-			graph->startFsmAction( actionOrd[i], actions[i].action->id );
+			graph->startFsmAction( actionOrd[i], actions[i].action->actionId );
 			afterOpMinimize( graph );
 			break;
 		case at_all:
-			graph->allTransAction( actionOrd[i], actions[i].action->id );
+			graph->allTransAction( actionOrd[i], actions[i].action->actionId );
 			break;
-		case at_fin:
-			graph->finFsmAction( actionOrd[i], actions[i].action->id );
+		case at_finish:
+			graph->finishFsmAction( actionOrd[i], actions[i].action->actionId );
 			break;
 		case at_leave:
-			graph->leaveFsmAction( actionOrd[i], actions[i].action->id );
+			graph->leaveFsmAction( actionOrd[i], actions[i].action->actionId );
 			break;
 
 		case at_start_gbl_error:
-			graph->startErrorAction( actionOrd[i], actions[i].action->id, 0 );
+			graph->startErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
 			afterOpMinimize( graph );
 			break;
 		case at_all_gbl_error:
-			graph->allErrorAction( actionOrd[i], actions[i].action->id, 0 );
+			graph->allErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
 			break;
 		case at_finish_gbl_error:
-			graph->finErrorAction( actionOrd[i], actions[i].action->id, 0 );
+			graph->finishErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
 			break;
 		case at_leave_gbl_error:
-			graph->leaveErrorAction( actionOrd[i], actions[i].action->id, 0 );
+			graph->leaveErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
 			break;
 		case at_start_finish_gbl_error:
-			graph->startErrorAction( actionOrd[i], actions[i].action->id, 0 );
-			graph->finErrorAction( actionOrd[i], actions[i].action->id, 0 );
+			graph->startErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
+			graph->finishErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
 			afterOpMinimize( graph );
 			break;
 		case at_finish_leave_gbl_error:
-			graph->finErrorAction( actionOrd[i], actions[i].action->id, 0 );
-			graph->leaveErrorAction( actionOrd[i], actions[i].action->id, 0 );
+			graph->finishErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
+			graph->leaveErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
 			break;
 
 		case at_start_local_error:
-			graph->startErrorAction( actionOrd[i], actions[i].action->id,
+			graph->startErrorAction( actionOrd[i], actions[i].action->actionId,
 					actions[i].localErrKey );
 			afterOpMinimize( graph );
 			break;
 		case at_all_local_error:
-			graph->allErrorAction( actionOrd[i], actions[i].action->id,
+			graph->allErrorAction( actionOrd[i], actions[i].action->actionId,
 					actions[i].localErrKey );
 			break;
 		case at_finish_local_error:
-			graph->finErrorAction( actionOrd[i], actions[i].action->id,
+			graph->finishErrorAction( actionOrd[i], actions[i].action->actionId,
 					actions[i].localErrKey );
 			break;
 		case at_leave_local_error:
-			graph->leaveErrorAction( actionOrd[i], actions[i].action->id,
+			graph->leaveErrorAction( actionOrd[i], actions[i].action->actionId,
 					actions[i].localErrKey );
 			break;
 		case at_start_finish_local_error:
-			graph->startErrorAction( actionOrd[i], actions[i].action->id,
+			graph->startErrorAction( actionOrd[i], actions[i].action->actionId,
 					actions[i].localErrKey );
-			graph->finErrorAction( actionOrd[i], actions[i].action->id,
+			graph->finishErrorAction( actionOrd[i], actions[i].action->actionId,
 					actions[i].localErrKey );
 			afterOpMinimize( graph );
 			break;
 		case at_finish_leave_local_error:
-			graph->finErrorAction( actionOrd[i], actions[i].action->id,
+			graph->finishErrorAction( actionOrd[i], actions[i].action->actionId,
 					actions[i].localErrKey );
-			graph->leaveErrorAction( actionOrd[i], actions[i].action->id,
+			graph->leaveErrorAction( actionOrd[i], actions[i].action->actionId,
 					actions[i].localErrKey );
 			break;
+
+		default:
+			/* Parser Prevents this case. */
+			break;
 		}
 	}
 }
@@ -504,19 +514,48 @@ void FactorWithAug::assignPriorities( FsmAp *graph, int *priorOrd )
 		case at_all:
 			graph->allTransPrior( priorOrd[i], &priorDescs[i] );
 			break;
-		case at_fin:
-			graph->finFsmPrior( priorOrd[i], &priorDescs[i] );
+		case at_finish:
+			graph->finishFsmPrior( priorOrd[i], &priorDescs[i] );
 			break;
 		case at_leave:
 			graph->leaveFsmPrior( priorOrd[i], &priorDescs[i] );
 			break;
-		case at_start_gbl_error: case at_all_gbl_error:
-		case at_finish_gbl_error: case at_leave_gbl_error:
-		case at_start_finish_gbl_error: case at_finish_leave_gbl_error:
-		case at_start_local_error: case at_all_local_error:
-		case at_finish_local_error: case at_leave_local_error:
-		case at_start_finish_local_error: case at_finish_leave_local_error:
-			/* Parser prevents this. */
+
+		default:
+			/* Parser Prevents this case. */
+			break;
+		}
+	}
+}
+
+/* Assign context. */
+void FactorWithAug::assignContext( FsmAp *graph )
+{
+	for ( int i = 0; i < contexts.length(); i++ ) {
+		switch ( contexts[i].type ) {
+		case at_start_context:
+			graph->startContext( contexts[i].contextId );
+			break;
+		case at_all_context:
+			graph->allContext( contexts[i].contextId );
+			break;
+		case at_finish_context:
+			graph->finishContext( contexts[i].contextId );
+			break;
+		case at_leave_context:
+			graph->leaveContext( contexts[i].contextId );
+			break;
+		case at_start_finish_context:
+			graph->startContext( contexts[i].contextId );
+			graph->finishContext( contexts[i].contextId );
+			break;
+		case at_finish_leave_context:
+			graph->finishContext( contexts[i].contextId );
+			graph->leaveContext( contexts[i].contextId );
+			break;
+
+		default:
+			/* Parser Prevents this case. */
 			break;
 		}
 	}
@@ -595,6 +634,9 @@ FsmAp *FactorWithAug::walk( ParseData *pd )
 		}
 	}
 
+	/* Assign context. */
+	assignContext( rtnVal );
+
 	if ( labels.length() > 0 ) {
 		/* Pop the names. */
 		pd->resetNameScope( nameFrame );
@@ -650,12 +692,13 @@ void FactorWithAug::resolveNameRefs( ParseData *pd )
 		EpsilonLink &link = epsilonLinks[ep];
 
 		if ( link.target.length() == 1 && strcmp( link.target.data[0], "final" ) == 0 ) {
-			/* Epsilon drawn to the implicit final state. */
+			/* Epsilon drawn to an implicit final state. An implicit final is
+			 * only available in join operations. */
 			link.resolvedName = pd->localNameScope->final;
 		}
 		else {
 			/* Do an search for the name. */
-			NameVect resolved;
+			NameSet resolved;
 			pd->resolveFrom( resolved, pd->localNameScope, link.target, 0 );
 			if ( resolved.length() > 0 ) {
 				/* Take the first one. */
@@ -663,7 +706,7 @@ void FactorWithAug::resolveNameRefs( ParseData *pd )
 				if ( resolved.length() > 1 ) {
 					/* Complain about the multiple references. */
 					error(link.loc) << "state reference " << link.target << 
-							" resolves to multiple states" << endl;
+							" resolves to multiple entry points" << endl;
 					errorStateLabels( resolved );
 				}
 			}
@@ -676,7 +719,7 @@ void FactorWithAug::resolveNameRefs( ParseData *pd )
 		else {
 			/* Complain, no recovery action, the epsilon op will ignore any
 			 * epsilon transitions whose names did not resolve. */
-			error(link.loc) << "could not resolve label " << link.target[0] << endl;
+			error(link.loc) << "could not resolve label " << link.target << endl;
 		}
 	}
 
