@@ -32,6 +32,7 @@ RedFsmAp::RedFsmAp( FsmAp *graph, bool complete )
 	keyOps(graph->keyOps),
 	nextActionId(0),
 	nextTransId(0),
+	nextStateId(0),
 	errState(0),
 	errTrans(0),
 	firstFinState(0),
@@ -90,18 +91,9 @@ void RedFsmAp::depthFirstOrdering()
 /* Assign state ids by appearance in the state list. */
 void RedFsmAp::sequentialStateIds()
 {
-	int nextId = 0;
+	nextStateId = 0;
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ )
-		st->id = nextId++;
-}
-
-/* Find the final state with the lowest id. */
-void RedFsmAp::findFirstFinState()
-{
-	for ( RedStateList::Iter st = stateList; st.lte(); st++ ) {
-		if ( st->isFinal && (firstFinState == 0 || st->id < firstFinState->id) )
-			firstFinState = st;
-	}
+		st->id = nextStateId++;
 }
 
 /* Stable sort the states by final state status. */
@@ -128,16 +120,25 @@ void RedFsmAp::sortStatesByFinal()
 void RedFsmAp::sortStateIdsByFinal()
 {
 	/* First pass to assign non final ids. */
-	int nextId = 0;
+	nextStateId = 0;
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ ) {
 		if ( ! st->isFinal ) 
-			st->id = nextId++;
+			st->id = nextStateId++;
 	}
 
 	/* Second pass to assign final ids. */
 	for ( RedStateList::Iter st = stateList; st.lte(); st++ ) {
 		if ( st->isFinal ) 
-			st->id = nextId++;
+			st->id = nextStateId++;
+	}
+}
+
+/* Find the final state with the lowest id. */
+void RedFsmAp::findFirstFinState()
+{
+	for ( RedStateList::Iter st = stateList; st.lte(); st++ ) {
+		if ( st->isFinal && (firstFinState == 0 || st->id < firstFinState->id) )
+			firstFinState = st;
 	}
 }
 
@@ -221,6 +222,37 @@ void RedFsmAp::chooseSingle()
 	}
 }
 
+void RedFsmAp::makeFlat()
+{
+	for ( RedStateList::Iter st = stateList; st.lte(); st++ ) {
+		if ( st->outRange.length() == 0 ) {
+			st->lowKey = st->highKey = 0;
+			st->transList = 0;
+		}
+		else {
+			st->lowKey = st->outRange[0].lowKey;
+			st->highKey = st->outRange[st->outRange.length()-1].highKey;
+			unsigned long long span = keyOps->span( st->lowKey, st->highKey );
+			st->transList = new RedTransAp*[ span ];
+			memset( st->transList, 0, sizeof(RedTransAp*)*span );
+			
+			for ( RedTransList::Iter trans = st->outRange; trans.lte(); trans++ ) {
+				unsigned long long base, trSpan;
+				base = keyOps->span( st->lowKey, trans->lowKey )-1;
+				trSpan = keyOps->span( trans->lowKey, trans->highKey );
+				for ( unsigned long long pos = 0; pos < trSpan; pos++ )
+					st->transList[base+pos] = trans->value;
+			}
+
+			/* Fill in the gaps with the default transition. */
+			for ( unsigned long long pos = 0; pos < span; pos++ ) {
+				if ( st->transList[pos] == 0 )
+					st->transList[pos] = st->defTrans;
+			}
+		}
+	}
+}
+
 
 /* A default transition has been picked, move it from the outRange to the
  * default pointer. */
@@ -425,7 +457,7 @@ RedStateAp *RedFsmAp::getErrorState()
 {
 	/* Check if we need to init the error trans. */
 	if ( errState == 0 ) {
-		errState = new RedStateAp( false, 0, 0 );
+		errState = new RedStateAp( false, 0 );
 		stateList.append( errState );
 	}
 	return errState;
@@ -597,7 +629,7 @@ RedTransAp *RedFsmAp::reduceTrans( TransAp *trans )
 	RedAction *action = 0;
 	if ( trans->actionTable.length() > 0 ) {
 		if ( actionMap.insert( trans->actionTable, &action ) )
-			action->id = nextActionId++;
+			action->actListId = nextActionId++;
 	}
 	
 	/* Get the target, if the target is null, use the error state. */
@@ -652,7 +684,7 @@ void RedFsmAp::reduceMachine()
 		RedAction *eofAction = 0;
 		if ( state->eofActionTable.length() > 0 ) {
 			if ( actionMap.insert( state->eofActionTable, &eofAction ) )
-				eofAction->id = nextActionId++;
+				eofAction->actListId = nextActionId++;
 		}
 
 		/* Clear the state's in lists. */
@@ -662,14 +694,23 @@ void RedFsmAp::reduceMachine()
 		/* Final state status. */
 		bool isFinal = state->isFinState();
 
+		/* Transfer the context set into a local var. */
+		ContextSet contextSet;
+		contextSet.shallowCopy( state->contextSet );
+		state->contextSet.abandon();
+
 		/* Convert the state to a RedStateAp. */
 		state->~StateAp();
-		RedStateAp *redState = new(state) RedStateAp( isFinal, eofAction, 0 );
+		RedStateAp *redState = new(state) RedStateAp( isFinal, eofAction );
 
-		/* Save off the data collected. */
+		/* Save the outlist. */
 		redState->outRange.shallowCopy( outRange );
 		outRange.abandon();
 
+		/* Save the context set. */
+		redState->contextSet.shallowCopy( contextSet );
+		contextSet.abandon();
+
 		/* Put the reduced state on the state list. */
 		stateList.append( redState );
 	}
