@@ -24,13 +24,6 @@
 #include "parsetree.h"
 #include "redfsm.h"
 
-#define SPEC_ANY_SINGLE  0x01
-#define SPEC_ANY_RANGE   0x02
-#define SPEC_EOF_ACTION  0x04
-
-/* Integer array line length. */
-#define IALL 8
-
 /* Pass init data to base class. */
 TabCodeGen::TabCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
@@ -38,76 +31,68 @@ TabCodeGen::TabCodeGen( char *fsmName, ParseData *parseData,
 	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-int TabCodeGen::makeStateSpec( RedStateAp *state )
+/* Determine if we should use indicies or not. */
+void TabCodeGen::calcIndexSize()
 {
-	int retSpec = 0;
-	/* Single index. */
-	if ( state->outSingle.length() > 0 )
-		retSpec |= SPEC_ANY_SINGLE;
-	/* Range index. */
-	if ( state->outRange.length() > 0 )
-		retSpec |= SPEC_ANY_RANGE;
-	/* Out action? */
-	if ( state->eofAction != 0 )
-		retSpec |= SPEC_EOF_ACTION;
-	return retSpec;
-}
+	int sizeWithInds = 0, sizeWithoutInds = 0;
 
-int TabCodeGen::getStateLen( int stateSpec )
-{
-	/* Basic len of 3 bytes. */
-	int stateLen = 3;
-	if ( stateSpec & SPEC_ANY_SINGLE )
-		stateLen += 1;
-	if ( stateSpec & SPEC_ANY_RANGE )
-		stateLen += 1;
-	if ( stateSpec & SPEC_EOF_ACTION )
-		stateLen += 1;
-	return stateLen;
-}
-
-void TabCodeGen::calcStatePositions()
-{
-	/* Since states are variable width, we must compute their locations. Loop
-	 * the states, keeping a calc of the length. */
-	int nextPosition = 0;
+	/* Calculate cost of using with indicies. */
 	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
-		/* Store the length. */
-		st->position = nextPosition;
-
-		/* Skip over the state. */
-		int stateSpec = makeStateSpec( st );
-		nextPosition += getStateLen( stateSpec );
+		int totalIndex = st->outSingle.length() + st->outRange.length() + 
+				(st->defTrans == 0 ? 0 : 1);
+		sizeWithInds += arrayTypeSize(maxIndex) * totalIndex;
 	}
+	sizeWithInds += arrayTypeSize(maxState) * redFsm->transSet.length();
+	if ( anyActions() )
+		sizeWithInds += arrayTypeSize(maxActionLoc) * redFsm->transSet.length();
+
+	/* Calculate the cost of not using indicies. */
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		int totalIndex = st->outSingle.length() + st->outRange.length() + 
+				(st->defTrans == 0 ? 0 : 1);
+		sizeWithoutInds += arrayTypeSize(maxState) * totalIndex;
+		if ( anyActions() )
+			sizeWithoutInds += arrayTypeSize(maxActionLoc) * totalIndex;
+	}
+
+	/* If using indicies reduces the size, use them. */
+	useIndicies = sizeWithInds < sizeWithoutInds;
 }
 
+
 std::ostream &TabCodeGen::STATE_OUT_ACTION( RedStateAp *state )
 {
 	/* This function is only called if there are any out actions, so need
 	 * not guard against there being none. */
-	out << state->eofAction->location+1;
+	int act = 0;
+	if ( state->eofAction != 0 )
+		act = state->eofAction->location+1;
+	out << act;
 	return out;
 }
 
 std::ostream &TabCodeGen::TRANS_ACTION( RedTransAp *trans )
 {
 	/* If there are actions, emit them. Otherwise emit zero. */
+	int act = 0;
 	if ( trans->action != 0 )
-		out << trans->action->location+1;
-	else
-		out << "0";
+		act = trans->action->location+1;
+	out << act;
 	return out;
 }
 
 std::ostream &TabCodeGen::FIRST_FINAL_STATE()
 {
-	out << redFsm->firstFinState->position;
+	if ( redFsm->firstFinState != 0 )
+		out << redFsm->firstFinState->id;
+	else
+		out << redFsm->nextStateId;
 	return out;
 }
 
 std::ostream &TabCodeGen::ERROR_STATE()
 {
-	out << redFsm->errState->position;
+	out << redFsm->errState->id;
 	return out;
 }
 
@@ -118,13 +103,13 @@ std::ostream &TabCodeGen::ACTION_SWITCH()
 		/* Write out referenced actions. */
 		if ( act->numTransRefs > 0 || act->numEofRefs > 0 ) {
 			/* Get the function data. Print the case label.  */
-			out << "\tcase " << act->id << ":\n";
+			out << "\tcase " << act->actionId << ":\n";
 
 			/* Write the preprocessor line info for going into the source file. */
 			out << "#line " << act->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
 
 			/* Write the block and close it off. */
-			out << "\t{"; ACTION(act, 0, false) << "} break;\n";
+			out << "\t{"; INLINE_LIST(act->inlineList, 0, false) << "} break;\n";
 		}
 	}
 
@@ -134,61 +119,96 @@ std::ostream &TabCodeGen::ACTION_SWITCH()
 	return out;
 }
 
-std::ostream &TabCodeGen::STATES()
+std::ostream &TabCodeGen::KEY_OFFSETS()
+{
+	out << "\t";
+	int totalStateNum = 0, curKeyOffset = 0;
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		/* Write the key offset. */
+		out << curKeyOffset;
+		if ( !st.last() ) {
+			out << ", ";
+			if ( ++totalStateNum % IALL == 0 )
+				out << "\n\t";
+		}
+
+		/* Move the key offset ahead. */
+		curKeyOffset += st->outSingle.length() + st->outRange.length()*2;
+	}
+	out << "\n";
+	return out;
+}
+
+
+std::ostream &TabCodeGen::INDEX_OFFSETS()
+{
+	out << "\t";
+	int totalStateNum = 0, curIndOffset = 0;
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		/* Write the index offset. */
+		out << curIndOffset;
+		if ( !st.last() ) {
+			out << ", ";
+			if ( ++totalStateNum % IALL == 0 )
+				out << "\n\t";
+		}
+
+		/* Move the index offset ahead. */
+		curIndOffset += st->outSingle.length() + st->outRange.length() + 1;
+	}
+	out << "\n";
+	return out;
+}
+
+std::ostream &TabCodeGen::SINGLE_LENS()
 {
-	/* Initial indent. */
 	out << "\t";
-	int curKeyOffset = 0, curIndOffset = 0;
 	int totalStateNum = 0;
 	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
-		/* Get the spec and write it and the length. */
-		int stateSpec = makeStateSpec( st );
-		int stateLen = getStateLen( stateSpec );
-		out << ( ( stateLen << 8 ) |  stateSpec ) << ", ";
-		if ( ++totalStateNum % IALL == 0 )
-			out << "\n\t";
-
-		/* Write the key offset. */
-		out << curKeyOffset << ", ";
-		if ( ++totalStateNum % IALL == 0 )
-			out << "\n\t";
-
-		/* Write the index offset. */
-		out << curIndOffset << ", ";
-		if ( ++totalStateNum % IALL == 0 )
-			out << "\n\t";
-
-		/* Emit length of the singles index. */
-		if ( stateSpec & SPEC_ANY_SINGLE ) {
-			out << st->outSingle.length() << ", ";
+		/* Write singles length. */
+		out << st->outSingle.length();
+		if ( !st.last() ) {
+			out << ", ";
 			if ( ++totalStateNum % IALL == 0 )
 				out << "\n\t";
 		}
-
-		/* Emit length of range index. */
-		if ( stateSpec & SPEC_ANY_RANGE ) {
-			out << st->outRange.length() << ", ";
-			if ( ++totalStateNum % IALL == 0 )
-				out << "\n\t";
-		}
-
-		/* If there is an out func, write it. */
-		if ( stateSpec & SPEC_EOF_ACTION ) {
-			STATE_OUT_ACTION( st ) << ", ";
-			if ( ++totalStateNum % IALL == 0 )
-				out << "\n\t";
-		}
-
-		/* Move the key and index offsets ahead. */
-		curKeyOffset += st->outSingle.length() + st->outRange.length()*2;
-		curIndOffset += st->outSingle.length() + st->outRange.length();
-		if ( st->defTrans != 0 )
-			curIndOffset += 1;
 	}
+	out << "\n";
+	return out;
+}
 
-	/* Output one last number so we don't have to figure out when the last
-	 * entry is and avoid writing a comma. */
-	out << 0 << "\n";
+std::ostream &TabCodeGen::RANGE_LENS()
+{
+	out << "\t";
+	int totalStateNum = 0;
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		/* Emit length of range index. */
+		out << st->outRange.length();
+		if ( !st.last() ) {
+			out << ", ";
+			if ( ++totalStateNum % IALL == 0 )
+				out << "\n\t";
+		}
+	}
+	out << "\n";
+	return out;
+}
+
+
+std::ostream &TabCodeGen::EOF_ACTIONS()
+{
+	out << "\t";
+	int totalStateNum = 0;
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		/* Write any eof action. */
+		STATE_OUT_ACTION(st);
+		if ( !st.last() ) {
+			out << ", ";
+			if ( ++totalStateNum % IALL == 0 )
+				out << "\n\t";
+		}
+	}
+	out << "\n";
 	return out;
 }
 
@@ -257,19 +277,80 @@ std::ostream &TabCodeGen::INDICIES()
 	return out;
 }
 
-std::ostream &TabCodeGen::INDEX_TYPE()
+std::ostream &TabCodeGen::TRANS_TARGS()
 {
-	if ( redFsm->transSet.length() <= 256 )
-		out << "unsigned char";
-	else if ( redFsm->transSet.length() <= 256*256 )
-		out << "unsigned short";
-	else 
-		out << "unsigned int";
+	int totalTrans = 0;
+	out << '\t';
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		/* Walk the singles. */
+		for ( RedTransList::Iter stel = st->outSingle; stel.lte(); stel++ ) {
+			RedTransAp *trans = stel->value;
+			out << trans->targ->id << ", ";
+			if ( ++totalTrans % IALL == 0 )
+				out << "\n\t";
+		}
+
+		/* Walk the ranges. */
+		for ( RedTransList::Iter rtel = st->outRange; rtel.lte(); rtel++ ) {
+			RedTransAp *trans = rtel->value;
+			out << trans->targ->id << ", ";
+			if ( ++totalTrans % IALL == 0 )
+				out << "\n\t";
+		}
+
+		/* The state's default target state. */
+		if ( st->defTrans != 0 ) {
+			RedTransAp *trans = st->defTrans;
+			out << trans->targ->id << ", ";
+			if ( ++totalTrans % IALL == 0 )
+				out << "\n\t";
+		}
+	}
+
+	/* Output one last number so we don't have to figure out when the last
+	 * entry is and avoid writing a comma. */
+	out << 0 << "\n";
 	return out;
 }
 
 
-std::ostream &TabCodeGen::TRANSITIONS()
+std::ostream &TabCodeGen::TRANS_ACTIONS()
+{
+	int totalTrans = 0;
+	out << '\t';
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		/* Walk the singles. */
+		for ( RedTransList::Iter stel = st->outSingle; stel.lte(); stel++ ) {
+			RedTransAp *trans = stel->value;
+			TRANS_ACTION( trans ) << ", ";
+			if ( ++totalTrans % IALL == 0 )
+				out << "\n\t";
+		}
+
+		/* Walk the ranges. */
+		for ( RedTransList::Iter rtel = st->outRange; rtel.lte(); rtel++ ) {
+			RedTransAp *trans = rtel->value;
+			TRANS_ACTION( trans ) << ", ";
+			if ( ++totalTrans % IALL == 0 )
+				out << "\n\t";
+		}
+
+		/* The state's default index goes next. */
+		if ( st->defTrans != 0 ) {
+			RedTransAp *trans = st->defTrans;
+			TRANS_ACTION( trans ) << ", ";
+			if ( ++totalTrans % IALL == 0 )
+				out << "\n\t";
+		}
+	}
+
+	/* Output one last number so we don't have to figure out when the last
+	 * entry is and avoid writing a comma. */
+	out << 0 << "\n";
+	return out;
+}
+
+std::ostream &TabCodeGen::TRANS_TARGS_WI()
 {
 	/* Transitions must be written ordered by their id. */
 	RedTransAp **transPtrs = new RedTransAp*[redFsm->transSet.length()];
@@ -278,26 +359,50 @@ std::ostream &TabCodeGen::TRANSITIONS()
 
 	/* Keep a count of the num of items in the array written. */
 	out << '\t';
-	int totalTransData = 0;
+	int totalStates = 0;
 	for ( int t = 0; t < redFsm->transSet.length(); t++ ) {
 		/* Write out the target state. */
 		RedTransAp *trans = transPtrs[t];
-		out << trans->targ->position << ", ";
-		if ( ++totalTransData % IALL == 0 )
-			out << "\n\t";
-
-		/* Write the function for the transition. */
-		TRANS_ACTION( trans ) << ", ";
-		if ( ++totalTransData % IALL == 0 )
-			out << "\n\t";
+		out << trans->targ->id;
+		if ( t < redFsm->transSet.length()-1 ) {
+			out << ", ";
+			if ( ++totalStates % IALL == 0 )
+				out << "\n\t";
+		}
 	}
-
-	out << "0" << "\n";
+	out << "\n";
 	delete[] transPtrs;
 	return out;
 }
 
 
+std::ostream &TabCodeGen::TRANS_ACTIONS_WI()
+{
+	/* Transitions must be written ordered by their id. */
+	RedTransAp **transPtrs = new RedTransAp*[redFsm->transSet.length()];
+	for ( TransApSet::Iter trans = redFsm->transSet; trans.lte(); trans++ )
+		transPtrs[trans->id] = trans;
+
+	/* Keep a count of the num of items in the array written. */
+	out << '\t';
+	int totalAct = 0;
+	for ( int t = 0; t < redFsm->transSet.length(); t++ ) {
+		/* Write the function for the transition. */
+		RedTransAp *trans = transPtrs[t];
+		TRANS_ACTION( trans );
+		if ( t < redFsm->transSet.length()-1 ) {
+			out << ", ";
+			if ( ++totalAct % IALL == 0 )
+				out << "\n\t";
+		}
+	}
+	out << "\n";
+	delete[] transPtrs;
+	return out;
+}
+
+
+
 std::ostream &TabCodeGen::BSEARCH()
 {
 	out <<
@@ -353,35 +458,32 @@ std::ostream &TabCodeGen::RANGE_BSEARCH()
 std::ostream &TabCodeGen::LOCATE_TRANS()
 {
 	out <<
-		"	_specs = *_pcs++;\n"
-		"	_keys = "; FSM_NAME() << "_k + *_pcs++;\n"
-		"	_inds = "; FSM_NAME() << "_i + *_pcs++;\n"
+		"	_keys = "; FSM_NAME() << "_k + "; FSM_NAME() << "_ko[_cs];\n"
+		"	_trans = "; FSM_NAME() << "_io[_cs];\n"
 		"\n"
-		"	if ( _specs & SPEC_ANY_SINGLE ) {\n"
-		"		int indsLen = *_pcs++;\n"
+		"	_slen = "; FSM_NAME() << "_sl[_cs];\n"
+		"	if ( _slen > 0 ) {\n"
 		"		"; ALPH_TYPE() << " *match = "; FSM_NAME() << 
-						"_bsearch( "; GET_KEY() << ", _keys, indsLen );\n"
+						"_bsearch( "; GET_KEY() << ", _keys, _slen );\n"
 		"		if ( match != 0 ) {\n"
-		"			_trans = "; FSM_NAME() << "_t + (_inds[match - _keys]<<1);\n"
+		"			_trans += (match - _keys);\n"
 		"			goto match;\n"
 		"		}\n"
-		"		_keys += indsLen;\n"
-		"		_inds += indsLen;\n"
+		"		_keys += _slen;\n"
+		"		_trans += _slen;\n"
 		"	}\n"
 		"\n"
-		"	if ( _specs & SPEC_ANY_RANGE ) {\n"
-		"		int indsLen = *_pcs++;\n"
+		"	_rlen = "; FSM_NAME() << "_rl[_cs];\n"
+		"	if ( _rlen > 0 ) {\n"
 		"		"; ALPH_TYPE() << " *match = "; FSM_NAME() << 
-						"_range_bsearch( "; GET_KEY() << ", _keys, (indsLen<<1) );\n"
+						"_range_bsearch( "; GET_KEY() << ", _keys, (_rlen<<1) );\n"
 		"		if ( match != 0 ) {\n"
-		"			_trans = "; FSM_NAME() << "_t + (_inds[(match - _keys)>>1]<<1);\n"
+		"			_trans += ((match - _keys)>>1);\n"
 		"			goto match;\n"
 		"		}\n"
-		"		_keys += (indsLen<<1);\n"
-		"		_inds += indsLen;\n"
+		"		_trans += _rlen;\n"
 		"	}\n"
-		"\n"
-		"	_trans = "; FSM_NAME() << "_t + ((*_inds)<<1);\n";
+		"\n";
 
 	return out;
 }
@@ -396,13 +498,15 @@ std::ostream &TabCodeGen::GOTO( NameInst *name, bool inFinish )
 {
 	/* Lookup the target. Always guaranteed to return just one target. */
 	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{_cs = " << targ->value->position << "; goto again;}";
+	out << "{_cs = " << targ->value->id << "; goto again;}";
 	return out;
 }
 
-std::ostream &TabCodeGen::GOTOE( char *stateExpr, bool inFinish )
+std::ostream &TabCodeGen::GOTOE( InlineItem *ilItem, bool inFinish )
 {
-	out << "{_cs = (" << stateExpr << "); goto again;}";
+	out << "{_cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << "); goto again;}";
 	return out;
 }
 
@@ -418,23 +522,150 @@ std::ostream &TabCodeGen::TARGS( bool inFinish, int targState )
 	return out;
 }
 
-std::ostream &TabCodeGen::GET_STATE( NameInst *name, bool inFinish )
+std::ostream &TabCodeGen::GET_ENTRY( NameInst *name, bool inFinish )
 {
 	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "("; FSM_NAME() << "_s+" << targ->value->position << ")";
+	out << targ->value->id;
 	return out;
 }
 
 std::ostream &TabCodeGen::NEXT( NameInst *name, bool inFinish )
 {
 	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "_cs = " << targ->value->position << ";";
+	out << "_cs = " << targ->value->id << ";";
 	return out;
 }
 
-std::ostream &TabCodeGen::NEXTE( char *stateExpr, bool inFinish )
+std::ostream &TabCodeGen::NEXTE( InlineItem *ilItem, bool inFinish )
 {
-	out << "_cs = (" << stateExpr << ");";
+	out << "_cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << ");";
+	return out;
+}
+
+std::ostream &TabCodeGen::CONTEXT( int contextId )
+{
+	/* Make a vect to fill with context indexed by state. */
+	int numVals = (redFsm->nextStateId>>3)+1;
+	unsigned char *vals = new unsigned char[numVals];
+	memset( vals, 0, sizeof(unsigned char)*numVals );
+
+	/* For each state where the context is present, set the bit. */
+	for ( RedStateList::Iter st = redFsm->stateList; st.lte(); st++ ) {
+		if ( st->contextSet.find( contextId ) )
+			vals[st->id>>3] |= 1 << (st->id&0x7);
+	}
+
+	/* Write out context info. */
+	for ( int i = 0; i < numVals; ) {
+		out << (unsigned int)vals[i];
+
+		i += 1;
+		if ( i < numVals ) {
+			out << ", ";
+			if ( i % IALL == 0 )
+				out << "\n\t";
+		}
+	}
+	out << "\n";
+
+	delete[] vals;
+	return out;
+}
+
+std::ostream &TabCodeGen::STATE_DATA()
+{
+	/* If there are any transtion functions then output the array. If there
+	 * are none, don't bother emitting an empty array that won't be used. */
+	if ( anyActions() ) {
+		out <<
+			"static "; ARRAY_TYPE(maxActArrItem) << " "; FSM_NAME() << "_a[] = {\n";
+			ACTIONS_ARRAY() <<
+			"};\n"
+			"\n";
+	}
+
+	out << 
+		"static "; ARRAY_TYPE(maxKeyOffset) << " "; FSM_NAME() << "_ko[] = {\n";
+		KEY_OFFSETS() <<
+		"};\n"
+		"\n";
+
+	out <<
+ 		"static "; ALPH_TYPE() << " "; FSM_NAME() << "_k[] = {\n";
+		KEYS() <<
+		"};\n"
+		"\n";
+
+	out << 
+		"static "; ARRAY_TYPE(maxSingleLen) << " "; FSM_NAME() << "_sl[] = {\n";
+		SINGLE_LENS() <<
+		"};\n"
+		"\n";
+
+	out << 
+		"static "; ARRAY_TYPE(maxRangeLen) << " "; FSM_NAME() << "_rl[] = {\n";
+		RANGE_LENS() <<
+		"};\n"
+		"\n";
+
+	out << 
+		"static "; ARRAY_TYPE(maxIndexOffset) << " "; FSM_NAME() << "_io[] = {\n";
+		INDEX_OFFSETS() <<
+		"};\n"
+		"\n";
+
+	if ( useIndicies ) {
+		out << 
+			"static "; ARRAY_TYPE(maxIndex) << " "; FSM_NAME() << "_i[] = {\n";
+			INDICIES() <<
+			"};\n"
+			"\n";
+
+		out << 
+			"static "; ARRAY_TYPE(maxState) << " "; FSM_NAME() << "_tt[] = {\n";
+			TRANS_TARGS_WI() <<
+			"};\n"
+			"\n";
+
+		if ( anyActions() ) {
+			out << 
+				"static "; ARRAY_TYPE(maxActionLoc) << " "; FSM_NAME() << "_ta[] = {\n";
+				TRANS_ACTIONS_WI() <<
+				"};\n"
+				"\n";
+		}
+	}
+	else {
+		out << 
+			"static "; ARRAY_TYPE(maxState) << " "; FSM_NAME() << "_tt[] = {\n";
+			TRANS_TARGS() <<
+			"};\n"
+			"\n";
+
+		if ( anyActions() ) {
+			out << 
+				"static "; ARRAY_TYPE(maxActionLoc) << " "; FSM_NAME() << "_ta[] = {\n";
+				TRANS_ACTIONS() <<
+				"};\n"
+				"\n";
+		}
+	}
+
+	if ( anyEofActions() ) {
+		out <<
+			"static "; ARRAY_TYPE(maxActionLoc) << " "; FSM_NAME() << "_ea[] = {\n";
+			EOF_ACTIONS() <<
+			"};\n"
+			"\n";
+	}
+
+	out <<
+		"static int "; FSM_NAME() << "_start = " 
+				<< redFsm->startState->id << ";\n"
+		"\n";
+
 	return out;
 }
 
@@ -450,7 +681,7 @@ std::ostream &CTabCodeGen::CALL( NameInst *name, int targState, bool inFinish )
 	/* Lookup the target. Always guaranteed to return just one target. */
 	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
 	out << "{fsm->_st[fsm->_top++] = _cs; _cs = " << 
-			targ->value->position << "; goto again;}";
+			targ->value->id << "; goto again;}";
 	return out;
 }
 
@@ -460,97 +691,31 @@ std::ostream &CTabCodeGen::RET( bool inFinish )
 	return out;
 }
 
-std::ostream &CTabCodeGen::CALLE( char *stateExpr, int targState, bool inFinish )
+std::ostream &CTabCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
 {
-	out << "{fsm->_st[fsm->_top++] = _cs; _cs = (" << stateExpr << "); goto again;}";
+	out << "{fsm->_st[fsm->_top++] = _cs; _cs = (";
+	INLINE_LIST( ilItem->children, targState, inFinish );
+	out << "); goto again;}";
 	return out;
 }
 
-/* Generate the header portion. */
-void CTabCodeGen::writeOutHeader()
-{
-	out << 
-		"/* Current state and user data. */\n"
-		"struct "; FSM_NAME() << "\n"
-		"{\n"
-		"	int _cs;\n";
-		STRUCT_DATA() <<
-		"};\n"
-		"\n"
-		"/* Init the fsm. */\n"
-		"int "; FSM_NAME() << "_init( struct "; FSM_NAME() << " *fsm );\n"
-		"\n"
-		"/* Execute some chunk of data. */\n"
-		"int "; FSM_NAME() << "_execute( struct "; FSM_NAME() << " *fsm, ";
-				EL_TYPE() << " *data, int len );\n"
-		"\n"
-		"/* Indicate to the fsm tha there is no more data. */\n"
-		"int "; FSM_NAME() << "_finish( struct "; FSM_NAME() << " *fsm );\n"
-		"\n";
-}
-
 void CTabCodeGen::writeOutCode()
 {
-	/* Need the state positions. */
-	calcStatePositions();
-	
-	/* State machine data. */
-	out << 
-		"#define SPEC_ANY_SINGLE  0x01\n"
-		"#define SPEC_ANY_RANGE   0x02\n"
-		"#define SPEC_EOF_ACTION  0x04\n"
-		"\n";
-
-	/* If there are any transtion functions then output the array. If there
-	 * are none, don't bother emitting an empty array that won't be used. */
-	if ( anyActions() ) {
-		out <<
-			"static int "; FSM_NAME() << "_a[] = {\n";
-			ACTIONS_ARRAY() <<
-			"};\n"
-			"\n";
-	}
-
-	/* Write the array of keys. */
-	out <<
- 		"static "; ALPH_TYPE() << " "; FSM_NAME() << "_k[] = {\n";
-		KEYS() <<
-		"};\n"
-		"\n";
-
-	/* Write the array of indicies. */
-	out << 
-		"static "; INDEX_TYPE() << " "; FSM_NAME() << "_i[] = {\n";
-		INDICIES() <<
-		"};\n"
-		"\n";
-
-	/* Write the array of states. */
-	out <<
-		"static int "; FSM_NAME() << "_s[] = {\n";
-		STATES() <<
-		"};\n"
-		"\n"
-		"static int "; FSM_NAME() << "_t[] = {\n";
-		TRANSITIONS() <<
-		"};\n"
-		"\n"
-		"static int "; FSM_NAME() << "_start = " 
-				<< redFsm->startState->position << ";\n"
-		"\n";
+	/* Write out state array data. */
+	STATE_DATA();
 
 	/* Init routine. */
 	out << 
 		"int "; FSM_NAME() << "_init( struct "; FSM_NAME() << " *fsm )\n"
 		"{\n"
-		"	fsm->_cs = "; FSM_NAME() << "_start;\n";
+		"	fsm->curs = "; FSM_NAME() << "_start;\n";
 	
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
 		out << "	fsm->_top = 0;\n";
 
 	INIT_CODE() <<
-		"	if ( fsm->_cs >= "; FIRST_FINAL_STATE() << " )\n"
+		"	if ( fsm->curs >= "; FIRST_FINAL_STATE() << " )\n"
 		"		return 1;\n"
 		"	return 0;\n"
 		"}\n"
@@ -569,27 +734,27 @@ void CTabCodeGen::writeOutCode()
 		"{\n"
 		"	"; EL_TYPE() << " *_p = _data-1;\n"
 		"	"; EL_TYPE() << " *_pe = _data+_len;\n"
-		"	int _cs = fsm->_cs, *_pcs";
+		"	int _slen, _rlen, _cs = fsm->curs";
 
 	if ( anyRegCurStateRef() )
 		out << ", _ps";
-	out << ";\n\tint _specs, *_trans";
+	out << ";\n	unsigned int _trans;\n";
 		
-	if ( anyActions() )
-		out << ", *_acts, _nacts";
+	if ( anyActions() ) {
+		out << "\t";
+		ARRAY_TYPE(maxActArrItem) << " *_acts, _nacts;\n";
+	}
 
 	out <<
-		";\n"
 		"	"; ALPH_TYPE() << " *_keys;\n"
-		"	"; INDEX_TYPE() << " *_inds;\n"
+		"	fsm->curs = -1;\n"
 		"\n";
 
 
 	out <<
 		"again:\n"
 		"	if ( ++_p == _pe )\n"
-		"		goto out;\n"
-		"	_pcs = "; FSM_NAME() << "_s + _cs;\n";
+		"		goto out;\n";
 
 	if ( anyEofActions() ) {
 		out <<
@@ -605,15 +770,20 @@ void CTabCodeGen::writeOutCode()
 	if ( anyRegCurStateRef() )
 		out << "	_ps = _cs;\n";
 
+	if ( useIndicies ) {
+		out << "	_trans = "; 
+		FSM_NAME() << "_i[_trans];\n";
+	}
+
 	out <<
-		"	_cs = *_trans++;\n"
+		"	_cs = "; FSM_NAME() << "_tt[_trans];\n"
 		"\n";
 
 	if ( anyActions() ) {
-		out << 
-			"	if ( *_trans == 0 )\n"
+		out <<
+			"	if ( "; FSM_NAME() << "_ta[_trans] == 0 )\n"
 			"		goto again;\n"
-			"	_acts = "; FSM_NAME() << "_a + *_trans;\n"
+			"	_acts = "; FSM_NAME() << "_a + "; FSM_NAME() << "_ta[_trans];\n"
 			"\n";
 
 		/* Only used if there are out actions. */
@@ -637,8 +807,8 @@ void CTabCodeGen::writeOutCode()
 	if ( anyEofActions() ) {
 		out <<
 			"eofActions:\n"
-			"	if ( *_pcs & SPEC_EOF_ACTION ) {\n"
-			"		_acts = "; FSM_NAME() << "_a+*(_pcs + (*_pcs>>8)-1);\n"
+			"	if ( "; FSM_NAME() << "_ea[_cs] != 0 ) {\n"
+			"		_acts = "; FSM_NAME() << "_a + "; FSM_NAME() << "_ea[_cs];\n"
 			"		_p = 0; _pe = (("; EL_TYPE() << "*)0)+1;\n"
 			"		goto execFuncs;\n"
 			"	}\n"
@@ -647,7 +817,7 @@ void CTabCodeGen::writeOutCode()
 
 	out <<
 		"out:\n"
-		"	fsm->_cs = _cs;\n";
+		"	fsm->curs = _cs;\n";
 
 	if ( redFsm->errState != 0 ) {
 		out << 
@@ -675,12 +845,12 @@ void CTabCodeGen::writeOutCode()
 	else {
 		if ( redFsm->errState != 0 ) {
 			out << 
-				"	if ( fsm->_cs == "; ERROR_STATE() << " )\n"
+				"	if ( fsm->curs == "; ERROR_STATE() << " )\n"
 				"		return -1;\n";
 		}
 
 		out <<
-			"	if ( fsm->_cs >= "; FIRST_FINAL_STATE() << " )\n"
+			"	if ( fsm->curs >= "; FIRST_FINAL_STATE() << " )\n"
 			"		return 1;\n"
 			"	return 0;\n";
 	}
@@ -689,126 +859,61 @@ void CTabCodeGen::writeOutCode()
 		"}\n"
 		"\n";
 
-	/* Cleanup after ourselves. */
-	out <<
-		"#undef SPEC_ANY_SINGLE\n"
-		"#undef SPEC_ANY_RANGE\n"
-		"#undef SPEC_EOF_ACTION\n"
-		"\n";
+	for ( ContextMap::Iter ctx = parseData->contextMap; ctx.lte(); ctx++ ) {
+		out << "unsigned char _"; FSM_NAME() << "_ctxdata_" << ctx->key << "[] = {\n\t";
+		CONTEXT( ctx->value->id );
+		out << "};\n\n";
+	}
 }
 
 /* Init base class. */
-CCTabCodeGen::CCTabCodeGen( char *fsmName, ParseData *parseData, 
+CppTabCodeGen::CppTabCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out ) 
 : 
 	TabCodeGen(fsmName, parseData, redFsm, out) 
 { }
 
-std::ostream &CCTabCodeGen::CALL( NameInst *name, int targState, bool inFinish )
+std::ostream &CppTabCodeGen::CALL( NameInst *name, int targState, bool inFinish )
 {
 	/* Lookup the target. Always guaranteed to return just one target. */
 	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
 	out << "{this->_st[this->_top++] = _cs; _cs = " << 
-			targ->value->position << "; goto again;}";
+			targ->value->id << "; goto again;}";
 	return out;
 }
 
-std::ostream &CCTabCodeGen::RET( bool inFinish )
+std::ostream &CppTabCodeGen::RET( bool inFinish )
 {
 	out << "{_cs = this->_st[--this->_top]; goto again;}";
 	return out;
 }
 
-std::ostream &CCTabCodeGen::CALLE( char *stateExpr, int targState, bool inFinish )
+std::ostream &CppTabCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
 {
 	/* Lookup the target. Always guaranteed to return just one target. */
-	out << "{this->_st[this->_top++] = _cs; _cs = (" << stateExpr << "); goto again;}";
+	out << "{this->_st[this->_top++] = _cs; _cs = (";
+	INLINE_LIST( ilItem->children, targState, inFinish );
+	out << "); goto again;}";
 	return out;
 }
 
-void CCTabCodeGen::writeOutHeader()
+void CppTabCodeGen::writeOutCode()
 {
-	out <<
-		"class "; FSM_NAME() << "\n"
-		"{\n"
-		"public:\n"
-		"	int _cs;\n"
-		"\n"
-		"	/* Initialize the machine for execution. */\n"
-		"	int init( );\n"
-		"\n"
-		"	/* Execute some chunk of data. */\n"
-		"	int execute( "; EL_TYPE() << " *data, int len );\n"
-		"\n"
-		"	/* Indicate to the fsm tha there is no more data. */\n"
-		"	int finish( );\n";
-		STRUCT_DATA() <<
-		"};\n"
-		"\n";
-}
-
-void CCTabCodeGen::writeOutCode()
-{
-	/* Need the state positions. */
-	calcStatePositions();
-	
-	/* State machine data. */
-	out << 
-		"#define SPEC_ANY_SINGLE  0x01\n"
-		"#define SPEC_ANY_RANGE   0x02\n"
-		"#define SPEC_EOF_ACTION  0x04\n"
-		"\n";
-
-	/* If there are any transtion functions then output the array. If there
-	 * are none, don't bother emitting an empty array that won't be used. */
-	if ( anyActions() ) {
-		out <<
-			"static int "; FSM_NAME() << "_a[] = {\n";
-			ACTIONS_ARRAY() <<
-			"};\n"
-			"\n";
-	}
-
-	/* Write the array of keys. */
-	out <<
-		"static "; ALPH_TYPE() << " "; FSM_NAME() << "_k[] = {\n";
-		KEYS() <<
-		"};\n"
-		"\n";
-
-	/* Write the array of indicies. */
-	out << 
-		"static "; INDEX_TYPE() << " "; FSM_NAME() << "_i[] = {\n";
-		INDICIES() <<
-		"};\n"
-		"\n";
-
-	/* Write the array of states. */
-	out <<
-		"static int "; FSM_NAME() << "_s[] = {\n";
-		STATES() <<
-		"};\n"
-		"\n"
-		"static int "; FSM_NAME() << "_t[] = {\n";
-		TRANSITIONS() <<
-		"};\n"
-		"\n"
-		"static int "; FSM_NAME() << "_start = " 
-				<< redFsm->startState->position << ";\n"
-		"\n";
+	/* Write out state array data. */
+	STATE_DATA();
 
 	/* Init routine. */
 	out << 
 		"int "; FSM_NAME() << "::init( )\n"
 		"{\n"
-		"	this->_cs = "; FSM_NAME() << "_start;\n";
+		"	this->curs = "; FSM_NAME() << "_start;\n";
 	
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
 		out << "	this->_top = 0;\n";
 
 	INIT_CODE() <<
-		"	if ( this->_cs >= "; FIRST_FINAL_STATE() << " )\n"
+		"	if ( this->curs >= "; FIRST_FINAL_STATE() << " )\n"
 		"		return 1;\n"
 		"	return 0;\n"
 		"}\n"
@@ -826,27 +931,27 @@ void CCTabCodeGen::writeOutCode()
 		"{\n"
 		"	"; EL_TYPE() << " *_p = _data-1;\n"
 		"	"; EL_TYPE() << " *_pe = _data+_len;\n"
-		"	int _cs = this->_cs, *_pcs";
+		"	int _slen, _rlen, _cs = this->curs";
 
 	if ( anyRegCurStateRef() )
 		out << ", _ps";
-	out << ";\n\tint _specs, *_trans";
+	out << ";\n	unsigned int _trans;\n";
 
-	if ( anyActions() )
-		out << ", *_acts, _nacts";
+	if ( anyActions() ) {
+		out << "\t";
+		ARRAY_TYPE(maxActArrItem) << " *_acts, _nacts;\n";
+	}
 
 	out <<
-		";\n"
 		"	"; ALPH_TYPE() << " *_keys;\n"
-		"	"; INDEX_TYPE() << " *_inds;\n"
+		"	this->curs = -1;\n"
 		"\n";
 	
 
 	out << 
 		"again:\n"
 		"	if ( ++_p == _pe )\n"
-		"		goto out;\n"
-		"	_pcs = "; FSM_NAME() << "_s + _cs;\n";
+		"		goto out;\n";
 
 	if ( anyEofActions() ) {
 		out << 
@@ -862,15 +967,20 @@ void CCTabCodeGen::writeOutCode()
 	if ( anyRegCurStateRef() )
 		out << "	_ps = _cs;\n";
 
+	if ( useIndicies ) {
+		out << "	_trans = "; 
+		FSM_NAME() << "_i[_trans];\n";
+	}
+
 	out <<
-		"	_cs = *_trans++;\n"
+		"	_cs = "; FSM_NAME() << "_tt[_trans];\n"
 		"\n";
 
 	if ( anyActions() ) {
 		out <<
-			"	if ( *_trans == 0 )\n"
+			"	if ( "; FSM_NAME() << "_ta[_trans] == 0 )\n"
 			"		goto again;\n"
-			"	_acts = "; FSM_NAME() << "_a + *_trans;\n"
+			"	_acts = "; FSM_NAME() << "_a + "; FSM_NAME() << "_ta[_trans];\n"
 			"\n";
 
 		/* Only used if there are out actions. */
@@ -894,8 +1004,8 @@ void CCTabCodeGen::writeOutCode()
 	if ( anyEofActions() ) {
 		out << 
 			"eofActions:\n"
-			"	if ( *_pcs & SPEC_EOF_ACTION ) {\n"
-			"		_acts = "; FSM_NAME() << "_a+*(_pcs + (*_pcs>>8)-1);\n"
+			"	if ( "; FSM_NAME() << "_ea[_cs] != 0 ) {\n"
+			"		_acts = "; FSM_NAME() << "_a + "; FSM_NAME() << "_ea[_cs];\n"
 			"		_p = 0; _pe = (("; EL_TYPE() << "*)0)+1;\n"
 			"		goto execFuncs;\n"
 			"	}\n"
@@ -904,7 +1014,7 @@ void CCTabCodeGen::writeOutCode()
 
 	out <<
 		"out:\n"
-		"	this->_cs = _cs;\n";
+		"	this->curs = _cs;\n";
 
 	if ( redFsm->errState != 0 ) {
 		out << 
@@ -932,12 +1042,12 @@ void CCTabCodeGen::writeOutCode()
 	else {
 		if ( redFsm->errState != 0 ) {
 			out << 
-				"	if ( this->_cs == "; ERROR_STATE() << " )\n"
+				"	if ( this->curs == "; ERROR_STATE() << " )\n"
 				"		return -1;\n";
 		}
 
 		out <<
-			"	if ( this->_cs >= "; FIRST_FINAL_STATE() << " )\n"
+			"	if ( this->curs >= "; FIRST_FINAL_STATE() << " )\n"
 			"		return 1;\n"
 			"	return 0;\n";
 	}
@@ -946,10 +1056,201 @@ void CCTabCodeGen::writeOutCode()
 		"}\n"
 		"\n";
 
-	/* Cleanup after ourselves. */
+	for ( ContextMap::Iter ctx = parseData->contextMap; ctx.lte(); ctx++ ) {
+		out << "unsigned char "; FSM_NAME() << "::_ctxdata_" << ctx->key << "[] = {\n\t";
+		CONTEXT( ctx->value->id );
+		out << "};\n\n";
+	}
+}
+
+/* Init base class. */
+ObjCTabCodeGen::ObjCTabCodeGen( char *fsmName, ParseData *parseData, 
+		RedFsmAp *redFsm, std::ostream &out ) 
+: 
+	TabCodeGen(fsmName, parseData, redFsm, out) 
+{ }
+
+std::ostream &ObjCTabCodeGen::CALL( NameInst *name, int targState, bool inFinish )
+{
+	/* Lookup the target. Always guaranteed to return just one target. */
+	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
+	out << "{self->_st[self->_top++] = _cs; _cs = " << 
+			targ->value->id << "; goto again;}";
+	return out;
+}
+
+std::ostream &ObjCTabCodeGen::RET( bool inFinish )
+{
+	out << "{_cs = self->_st[--self->_top]; goto again;}";
+	return out;
+}
+
+std::ostream &ObjCTabCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
+{
+	/* Lookup the target. Always guaranteed to return just one target. */
+	out << "{self->_st[self->_top++] = _cs; _cs = (";
+	INLINE_LIST( ilItem->children, targState, inFinish );
+	out << "); goto again;}";
+	return out;
+}
+
+void ObjCTabCodeGen::writeOutCode()
+{
+	/* Write out state array data. */
+	STATE_DATA();
+
 	out <<
-		"#undef SPEC_ANY_SINGLE\n"
-		"#undef SPEC_ANY_RANGE\n"
-		"#undef SPEC_EOF_ACTION\n"
+		"@implementation "; FSM_NAME() << "\n\n"
+
+		"- (int) initFsm;\n"
+		"{\n"
+		"\tself->curs = "; FSM_NAME() << "_start;\n";
+
+	/* If there are any calls, then the stack top needs initialization. */
+	if ( anyActionCalls() || anyActionRets() )
+		out << "\tself->_top = 0;\n\n";
+
+	INIT_CODE() <<
+		"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n"
+		"}\n"
 		"\n";
+
+	/* The binary search for singles. */
+	BSEARCH() << "\n";
+
+	/* The binary search for a range. */
+	RANGE_BSEARCH() << "\n";
+
+	/* Execution function. */
+	out <<
+		"- (int) executeWithData:("; EL_TYPE() << " *)_data len:(int)_len;\n"
+		"{\n"
+		"\t"; EL_TYPE() << " *_p = _data - 1;\n"
+		"\t"; EL_TYPE() << " *_pe = _data + _len;\n"
+		"\tint _slen, _rlen, _cs = self->curs";
+
+	if ( anyRegCurStateRef() )
+		out << ", _ps";
+	out << ";\n	unsigned int _trans;\n";
+
+	if ( anyActions() ) {
+		out << "\t";
+		ARRAY_TYPE(maxActArrItem) << " *_acts, _nacts;\n";
+	}
+
+	out <<
+		"\t"; ALPH_TYPE() << " *_keys;\n"
+		"\tself->curs = -1;\n"
+		"\n";
+	
+	out << 
+		"again:\n"
+			"\tif ( ++_p == _pe )\n"
+				"\t\tgoto out;\n";
+
+	if ( anyEofActions() ) {
+		out << 
+			"\tif ( _p == 0 )\n"
+				"\t\tgoto eofActions;\n"
+			"\n";
+	}
+
+	LOCATE_TRANS() <<
+		"\n"
+		"match:\n";
+
+	if ( anyRegCurStateRef() )
+		out << "\t_ps = _cs;\n";
+
+	if ( useIndicies ) {
+		out << "\t_trans = "; 
+		FSM_NAME() << "_i[_trans];\n";
+	}
+
+	out <<
+		"\t_cs = "; FSM_NAME() << "_tt[_trans];\n"
+		"\n";
+
+	if ( anyActions() ) {
+		out <<
+			"\tif ( "; FSM_NAME() << "_ta[_trans] == 0 )\n"
+				"\t\tgoto again;\n"
+			"\t_acts = "; FSM_NAME() << "_a + "; FSM_NAME() << "_ta[_trans];\n"
+			"\n";
+
+		/* Only used if there are out actions. */
+		if ( anyEofActions() )
+			out << "execFuncs:\n";
+
+		out <<
+			"\t_nacts = *_acts++;\n"
+			"\twhile ( _nacts-- > 0 ) {\n"
+				"\t\tswitch ( *_acts++ ) {\n";
+				ACTION_SWITCH() <<
+				"\t\t}\n"
+			"\t}\n"
+			"\n";
+	}
+
+	out <<
+		"\tgoto again;\n"
+		"\n";
+	
+	if ( anyEofActions() ) {
+		out << 
+			"eofActions:\n"
+				"\tif ( "; FSM_NAME() << "_ea[_cs] != 0 ) {\n"
+					"\t\t_acts = "; FSM_NAME() << "_a + "; FSM_NAME() << "_ea[_cs];\n"
+					"\t\t_p = 0; _pe = (("; EL_TYPE() << "*)0)+1;\n"
+					"\t\tgoto execFuncs;\n"
+				"\t}\n"
+			"\n";
+	}
+
+	out <<
+		"out:\n"
+			"\tself->curs = _cs;\n";
+
+	if ( redFsm->errState != 0 ) {
+		out << 
+			"\tif ( _cs == "; ERROR_STATE() << " ) return -1;\n";
+	}
+
+	out <<
+			"\treturn ( _cs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n"
+		"}\n"
+		"\n";
+
+	out <<
+		"- (int) finish;\n"
+		"{\n";
+	
+	if ( anyEofActions() ) {
+		/* May need to execute code in the action loop. Must use special
+		 * params to execute func. */
+		out <<
+			"\treturn [self executeWithData:("; EL_TYPE() << "*)0 len:1];\n";
+	}
+	else {
+		if ( redFsm->errState != 0 ) {
+			out << 
+			"\tif ( self->curs == "; ERROR_STATE() << " ) return -1;\n";
+		}
+
+		out <<
+			"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n";
+	}
+
+	out << 
+		"}\n"
+		"\n";
+
+	for ( ContextMap::Iter ctx = parseData->contextMap; ctx.lte(); ctx++ ) {
+		out << "unsigned char "; FSM_NAME() << "::_ctxdata_" << ctx->key << "[] = {\n\t";
+		CONTEXT( ctx->value->id );
+		out << "};\n\n";
+	}
+	
+	out <<
+		"@end\n\n";
 }
